@@ -660,9 +660,9 @@ rspamd_protocol_handle_headers(struct rspamd_task *task,
 		IF_HEADER(USER_HEADER)
 		{
 			/*
-					 * We must ignore User header in case of spamc, as SA has
-					 * different meaning of this header
-					 */
+								 * We must ignore User header in case of spamc, as SA has
+								 * different meaning of this header
+								 */
 			msg_debug_protocol("read user header, value: %T", hv_tok);
 			if (!RSPAMD_TASK_IS_SPAMC(task)) {
 				task->auth_user = rspamd_mempool_ftokdup(task->task_pool,
@@ -708,15 +708,18 @@ rspamd_protocol_handle_headers(struct rspamd_task *task,
 				task->flags |= RSPAMD_TASK_FLAG_NO_LOG;
 			}
 		}
+		IF_HEADER(LOG_TAG_HEADER)
+		{
+			msg_debug_protocol("read log-tag header, value: %T", hv_tok);
+			/* Ensure that a tag is valid */
+			if (rspamd_fast_utf8_validate(hv_tok->begin, hv_tok->len) == 0) {
+				memcpy(task->task_pool->tag.uid, hv_tok->begin,
+					   MIN(hv_tok->len, sizeof(task->task_pool->tag.uid)));
+			}
+		}
 		break;
 	case 'm':
 	case 'M':
-		IF_HEADER(MLEN_HEADER)
-		{
-			msg_debug_protocol("read message length header, value: %T",
-							   hv_tok);
-			task->protocol_flags |= RSPAMD_TASK_PROTOCOL_FLAG_HAS_CONTROL;
-		}
 		IF_HEADER(MTA_TAG_HEADER)
 		{
 			char *mta_tag;
@@ -752,9 +755,9 @@ rspamd_protocol_handle_headers(struct rspamd_task *task,
 	default:
 		msg_debug_protocol("generic header: %T", hn_tok);
 		break;
-			}
+				}
 
-			rspamd_task_add_request_header (task, hn_tok, hv_tok);
+				rspamd_task_add_request_header (task, hn_tok, hv_tok);
 }
 }); /* End of kh_foreach_value */
 
@@ -773,120 +776,6 @@ if (!has_ip) {
 return TRUE;
 }
 
-#define BOOL_TO_FLAG(val, flags, flag) \
-	do {                               \
-		if ((val)) (flags) |= (flag);  \
-		else                           \
-			(flags) &= ~(flag);        \
-	} while (0)
-
-gboolean
-rspamd_protocol_parse_task_flags(rspamd_mempool_t *pool,
-								 const ucl_object_t *obj,
-								 gpointer ud,
-								 struct rspamd_rcl_section *section,
-								 GError **err)
-{
-	struct rspamd_rcl_struct_parser *pd = ud;
-	int *target;
-	const char *key;
-	gboolean value;
-
-	target = (int *) (((char *) pd->user_struct) + pd->offset);
-	key = ucl_object_key(obj);
-	value = ucl_object_toboolean(obj);
-
-	if (key != NULL) {
-		if (g_ascii_strcasecmp(key, "pass_all") == 0) {
-			BOOL_TO_FLAG(value, *target, RSPAMD_TASK_FLAG_PASS_ALL);
-		}
-		else if (g_ascii_strcasecmp(key, "no_log") == 0) {
-			BOOL_TO_FLAG(value, *target, RSPAMD_TASK_FLAG_NO_LOG);
-		}
-	}
-
-	return TRUE;
-}
-
-static struct rspamd_rcl_sections_map *control_parser = NULL;
-
-RSPAMD_CONSTRUCTOR(rspamd_protocol_control_parser_ctor)
-{
-
-	struct rspamd_rcl_section *sub = rspamd_rcl_add_section(&control_parser, NULL,
-															"*",
-															NULL,
-															NULL,
-															UCL_OBJECT,
-															FALSE,
-															TRUE);
-	/* Default handlers */
-	rspamd_rcl_add_default_handler(sub,
-								   "ip",
-								   rspamd_rcl_parse_struct_addr,
-								   G_STRUCT_OFFSET(struct rspamd_task, from_addr),
-								   0,
-								   NULL);
-	rspamd_rcl_add_default_handler(sub,
-								   "from",
-								   rspamd_rcl_parse_struct_mime_addr,
-								   G_STRUCT_OFFSET(struct rspamd_task, from_envelope),
-								   0,
-								   NULL);
-	rspamd_rcl_add_default_handler(sub,
-								   "rcpt",
-								   rspamd_rcl_parse_struct_mime_addr,
-								   G_STRUCT_OFFSET(struct rspamd_task, rcpt_envelope),
-								   0,
-								   NULL);
-	rspamd_rcl_add_default_handler(sub,
-								   "helo",
-								   rspamd_rcl_parse_struct_string,
-								   G_STRUCT_OFFSET(struct rspamd_task, helo),
-								   0,
-								   NULL);
-	rspamd_rcl_add_default_handler(sub,
-								   "user",
-								   rspamd_rcl_parse_struct_string,
-								   G_STRUCT_OFFSET(struct rspamd_task, auth_user),
-								   0,
-								   NULL);
-	rspamd_rcl_add_default_handler(sub,
-								   "pass_all",
-								   rspamd_protocol_parse_task_flags,
-								   G_STRUCT_OFFSET(struct rspamd_task, flags),
-								   0,
-								   NULL);
-	rspamd_rcl_add_default_handler(sub,
-								   "json",
-								   rspamd_protocol_parse_task_flags,
-								   G_STRUCT_OFFSET(struct rspamd_task, flags),
-								   0,
-								   NULL);
-}
-
-RSPAMD_DESTRUCTOR(rspamd_protocol_control_parser_dtor)
-{
-	rspamd_rcl_sections_free(control_parser);
-}
-
-gboolean
-rspamd_protocol_handle_control(struct rspamd_task *task,
-							   const ucl_object_t *control)
-{
-	GError *err = NULL;
-
-	if (!rspamd_rcl_parse(control_parser, task->cfg, task, task->task_pool,
-						  control, &err)) {
-		msg_warn_protocol("cannot parse control block: %e", err);
-		g_error_free(err);
-
-		return FALSE;
-	}
-
-	return TRUE;
-}
-
 gboolean
 rspamd_protocol_handle_request(struct rspamd_task *task,
 							   struct rspamd_http_message *msg)
@@ -2202,6 +2091,7 @@ void rspamd_protocol_write_reply(struct rspamd_task *task, ev_tstamp timeout, st
 			memcpy(&stat_copy, srv->stat, sizeof(stat_copy));
 			output = rspamd_metrics_to_prometheus_string(
 				rspamd_worker_metrics_object(srv->cfg, &stat_copy, now - srv->start_time));
+			rspamd_printf_fstring(&output, "# EOF\n");
 			rspamd_http_message_set_body_from_fstring_steal(msg, output);
 			ctype = "application/openmetrics-text; version=1.0.0; charset=utf-8";
 			break;
