@@ -1,11 +1,11 @@
-/*-
- * Copyright 2016 Vsevolod Stakhov
+/*
+ * Copyright 2024 Vsevolod Stakhov
  *
  * Licensed under the Apache License, Version 2.0 (the "License");
  * you may not use this file except in compliance with the License.
  * You may obtain a copy of the License at
  *
- *   http://www.apache.org/licenses/LICENSE-2.0
+ *    http://www.apache.org/licenses/LICENSE-2.0
  *
  * Unless required by applicable law or agreed to in writing, software
  * distributed under the License is distributed on an "AS IS" BASIS,
@@ -31,6 +31,11 @@
 #include "hyperscan_tools.h"
 #endif
 
+#define msg_debug_control(...) rspamd_conditional_debug_fast(NULL, NULL,                             \
+															 rspamd_control_log_id, "control", NULL, \
+															 RSPAMD_LOG_FUNC,                        \
+															 __VA_ARGS__)
+
 static ev_tstamp io_timeout = 30.0;
 static ev_tstamp worker_io_timeout = 0.5;
 
@@ -40,23 +45,27 @@ struct rspamd_control_reply_elt {
 	struct rspamd_control_reply reply;
 	struct rspamd_io_ev ev;
 	struct ev_loop *event_loop;
+	struct rspamd_worker *worker;
 	GQuark wrk_type;
 	pid_t wrk_pid;
+	rspamd_ev_cb handler;
 	gpointer ud;
-	gint attached_fd;
+	int attached_fd;
+	bool sent;
+	struct rspamd_control_command cmd;
 	GHashTable *pending_elts;
 	struct rspamd_control_reply_elt *prev, *next;
 };
 
 struct rspamd_control_session {
-	gint fd;
+	int fd;
 	struct ev_loop *event_loop;
 	struct rspamd_main *rspamd_main;
 	struct rspamd_http_connection *conn;
 	struct rspamd_control_command cmd;
 	struct rspamd_control_reply_elt *replies;
 	rspamd_inet_addr_t *addr;
-	guint replies_remain;
+	unsigned int replies_remain;
 	gboolean is_reply;
 };
 
@@ -76,20 +85,12 @@ static const struct rspamd_control_cmd_match {
 };
 
 static void rspamd_control_ignore_io_handler(int fd, short what, void *ud);
+static void rspamd_control_stop_pending(struct rspamd_control_reply_elt *elt);
 
-static void
-rspamd_control_stop_pending(struct rspamd_control_reply_elt *elt)
-{
-	GHashTable *htb;
-	/* It stops event and frees hash */
-	htb = elt->pending_elts;
-	g_hash_table_remove(elt->pending_elts, elt);
-	/* Release hash reference */
-	g_hash_table_unref(htb);
-}
+INIT_LOG_MODULE(control)
 
 void rspamd_control_send_error(struct rspamd_control_session *session,
-							   gint code, const gchar *error_msg, ...)
+							   int code, const char *error_msg, ...)
 {
 	struct rspamd_http_message *msg;
 	rspamd_fstring_t *reply;
@@ -165,10 +166,10 @@ rspamd_control_write_reply(struct rspamd_control_session *session)
 {
 	ucl_object_t *rep, *cur, *workers;
 	struct rspamd_control_reply_elt *elt;
-	gchar tmpbuf[64];
-	gdouble total_utime = 0, total_systime = 0;
+	char tmpbuf[64];
+	double total_utime = 0, total_systime = 0;
 	struct ucl_parser *parser;
-	guint total_conns = 0;
+	unsigned int total_conns = 0;
 
 	rep = ucl_object_typed_new(UCL_OBJECT);
 	workers = ucl_object_typed_new(UCL_OBJECT);
@@ -287,11 +288,11 @@ rspamd_control_write_reply(struct rspamd_control_session *session)
 }
 
 static void
-rspamd_control_wrk_io(gint fd, short what, gpointer ud)
+rspamd_control_wrk_io(int fd, short what, gpointer ud)
 {
 	struct rspamd_control_reply_elt *elt = ud;
 	struct rspamd_control_session *session;
-	guchar fdspace[CMSG_SPACE(sizeof(int))];
+	unsigned char fdspace[CMSG_SPACE(sizeof(int))];
 	struct iovec iov;
 	struct msghdr msg;
 	gssize r;
@@ -357,14 +358,120 @@ void rspamd_pending_control_free(gpointer p)
 {
 	struct rspamd_control_reply_elt *rep_elt = (struct rspamd_control_reply_elt *) p;
 
-	rspamd_ev_watcher_stop(rep_elt->event_loop, &rep_elt->ev);
+	if (rep_elt->sent) {
+		rspamd_ev_watcher_stop(rep_elt->event_loop, &rep_elt->ev);
+	}
+	else if (rep_elt->attached_fd != -1) {
+		/* Only for non-sent requests! */
+		close(rep_elt->attached_fd);
+	}
+
+	g_hash_table_unref(rep_elt->pending_elts);
 	g_free(rep_elt);
 }
 
+static inline void
+rspamd_control_fill_msghdr(struct rspamd_control_command *cmd,
+						   int attached_fd, struct msghdr *msg,
+						   struct iovec *iov)
+{
+	struct cmsghdr *cmsg;
+	unsigned char fdspace[CMSG_SPACE(sizeof(int))];
+
+	memset(msg, 0, sizeof(*msg));
+
+	/* Attach fd to the message */
+	if (attached_fd != -1) {
+		memset(fdspace, 0, sizeof(fdspace));
+		msg->msg_control = fdspace;
+		msg->msg_controllen = sizeof(fdspace);
+		cmsg = CMSG_FIRSTHDR(msg);
+		cmsg->cmsg_level = SOL_SOCKET;
+		cmsg->cmsg_type = SCM_RIGHTS;
+		cmsg->cmsg_len = CMSG_LEN(sizeof(int));
+		memcpy(CMSG_DATA(cmsg), &attached_fd, sizeof(int));
+	}
+
+	iov->iov_base = cmd;
+	iov->iov_len = sizeof(*cmd);
+	msg->msg_iov = iov;
+	msg->msg_iovlen = 1;
+}
+
+static void
+rspamd_control_stop_pending(struct rspamd_control_reply_elt *elt)
+{
+	GHashTable *htb;
+	struct rspamd_main *rspamd_main;
+	gsize pending;
+
+	/* It stops event and frees hash */
+	htb = elt->pending_elts;
+
+	pending = g_hash_table_size(htb);
+	msg_debug_control("stop pending for %P(%s), %d events pending", elt->wrk_pid,
+					  g_quark_to_string(elt->wrk_type),
+					  (int) pending);
+
+	if (elt->worker->state != rspamd_worker_state_terminating && pending != 0) {
+		/* Invoke another event from the queue */
+		GHashTableIter it;
+		gpointer k, v;
+
+		g_hash_table_iter_init(&it, elt->pending_elts);
+
+		while (g_hash_table_iter_next(&it, &k, &v)) {
+			struct rspamd_control_reply_elt *cur = v;
+
+			if (!cur->sent) {
+				struct msghdr msg;
+				struct iovec iov;
+
+				rspamd_main = cur->worker->srv;
+				rspamd_control_fill_msghdr(&cur->cmd, cur->attached_fd, &msg, &iov);
+				ssize_t r = sendmsg(cur->worker->control_pipe[0], &msg, 0);
+
+				if (r == sizeof(cur->cmd)) {
+					msg_debug_control("restarting pending event for %P(%s), %d events pending",
+									  cur->wrk_pid,
+									  g_quark_to_string(cur->wrk_type),
+									  (int) pending - 1);
+					rspamd_ev_watcher_init(&cur->ev,
+										   cur->worker->control_pipe[0],
+										   EV_READ, cur->handler,
+										   cur);
+					rspamd_ev_watcher_start(cur->event_loop,
+											&cur->ev, worker_io_timeout);
+					cur->sent = true;
+					if (cur->attached_fd != -1) {
+						/* Since `sendmsg` performs `dup` for us, we need to remove our own descriptor */
+						close(cur->attached_fd);
+						cur->attached_fd = -1;
+					}
+
+					break; /* Exit the outer loop as we have invoked something */
+				}
+				else {
+					msg_err_main("cannot write command %d to the worker %P(%s), fd: %d: %s",
+								 (int) cur->cmd.type,
+								 cur->wrk_pid,
+								 g_quark_to_string(cur->wrk_type),
+								 cur->worker->control_pipe[0],
+								 strerror(errno));
+					g_hash_table_remove(elt->pending_elts, cur);
+				}
+			}
+		}
+	}
+
+	/* Remove from hash and performs the cleanup */
+	g_hash_table_remove(elt->pending_elts, elt);
+}
+
 static struct rspamd_control_reply_elt *
 rspamd_control_broadcast_cmd(struct rspamd_main *rspamd_main,
 							 struct rspamd_control_command *cmd,
-							 gint attached_fd,
+							 int attached_fd,
 							 rspamd_ev_cb handler,
 							 gpointer ud,
 							 pid_t except_pid)
@@ -373,10 +480,6 @@ rspamd_control_broadcast_cmd(struct rspamd_main *rspamd_main,
 	struct rspamd_worker *wrk;
 	struct rspamd_control_reply_elt *rep_elt, *res = NULL;
 	gpointer k, v;
-	struct msghdr msg;
-	struct cmsghdr *cmsg;
-	struct iovec iov;
-	guchar fdspace[CMSG_SPACE(sizeof(int))];
 	gssize r;
 
 	g_hash_table_iter_init(&it, rspamd_main->workers);
@@ -398,51 +501,89 @@ rspamd_control_broadcast_cmd(struct rspamd_main *rspamd_main,
 			continue;
 		}
 
-		memset(&msg, 0, sizeof(msg));
+		rep_elt = g_malloc0(sizeof(*rep_elt));
+		rep_elt->worker = wrk;
+		rep_elt->wrk_pid = wrk->pid;
+		rep_elt->wrk_type = wrk->type;
+		rep_elt->event_loop = rspamd_main->event_loop;
+		rep_elt->ud = ud;
+		rep_elt->handler = handler;
+		memcpy(&rep_elt->cmd, cmd, sizeof(*cmd));
+		rep_elt->sent = false;
+		rep_elt->attached_fd = -1;
 
-		/* Attach fd to the message */
-		if (attached_fd != -1) {
-			memset(fdspace, 0, sizeof(fdspace));
-			msg.msg_control = fdspace;
-			msg.msg_controllen = sizeof(fdspace);
-			cmsg = CMSG_FIRSTHDR(&msg);
-			cmsg->cmsg_level = SOL_SOCKET;
-			cmsg->cmsg_type = SCM_RIGHTS;
-			cmsg->cmsg_len = CMSG_LEN(sizeof(int));
-			memcpy(CMSG_DATA(cmsg), &attached_fd, sizeof(int));
-		}
+		if (g_hash_table_size(wrk->control_events_pending) == 0) {
+			/* We can send command */
+			struct msghdr msg;
+			struct iovec iov;
 
-		iov.iov_base = cmd;
-		iov.iov_len = sizeof(*cmd);
-		msg.msg_iov = &iov;
-		msg.msg_iovlen = 1;
+			rspamd_control_fill_msghdr(cmd, attached_fd, &msg, &iov);
+			r = sendmsg(wrk->control_pipe[0], &msg, 0);
 
-		r = sendmsg(wrk->control_pipe[0], &msg, 0);
+			if (r == sizeof(*cmd)) {
+				rspamd_ev_watcher_init(&rep_elt->ev,
+									   wrk->control_pipe[0],
+									   EV_READ, handler,
+									   rep_elt);
+				rspamd_ev_watcher_start(rspamd_main->event_loop,
+										&rep_elt->ev, worker_io_timeout);
+				rep_elt->sent = true;
+				rep_elt->pending_elts = g_hash_table_ref(wrk->control_events_pending);
+				g_hash_table_insert(wrk->control_events_pending, rep_elt, rep_elt);
 
-		if (r == sizeof(*cmd)) {
-			rep_elt = g_malloc0(sizeof(*rep_elt));
-			rep_elt->wrk_pid = wrk->pid;
-			rep_elt->wrk_type = wrk->type;
-			rep_elt->event_loop = rspamd_main->event_loop;
-			rep_elt->ud = ud;
-			rep_elt->pending_elts = g_hash_table_ref(wrk->control_events_pending);
-			rspamd_ev_watcher_init(&rep_elt->ev,
-								   wrk->control_pipe[0],
-								   EV_READ, handler,
-								   rep_elt);
-			rspamd_ev_watcher_start(rspamd_main->event_loop,
-									&rep_elt->ev, worker_io_timeout);
-			g_hash_table_insert(wrk->control_events_pending, rep_elt, rep_elt);
-
-			DL_APPEND(res, rep_elt);
+				DL_APPEND(res, rep_elt);
+				msg_debug_control("sent command %d to the worker %P(%s), fd: %d",
+								  (int) cmd->type,
+								  wrk->pid,
+								  g_quark_to_string(wrk->type),
+								  wrk->control_pipe[0]);
+			}
+			else {
+				msg_err_main("cannot write command %d to the worker %P(%s), fd: %d: %s",
+							 (int) cmd->type,
+							 wrk->pid,
+							 g_quark_to_string(wrk->type),
+							 wrk->control_pipe[0],
+							 strerror(errno));
+				g_free(rep_elt);
+			}
 		}
 		else {
-			msg_err_main("cannot write command %d(%z) to the worker %P(%s), fd: %d: %s",
-						 (int) cmd->type, iov.iov_len,
-						 wrk->pid,
-						 g_quark_to_string(wrk->type),
-						 wrk->control_pipe[0],
-						 strerror(errno));
+			/* We need to wait till the last command is processed, or it will mess up all serialization */
+			msg_debug_control("pending event for %P(%s), %d events pending",
+							  wrk->pid,
+							  g_quark_to_string(wrk->type),
+							  (int) g_hash_table_size(wrk->control_events_pending));
+			rep_elt->pending_elts = g_hash_table_ref(wrk->control_events_pending);
+			/*
+			 * Here are dragons:
+			 * If we have a descriptor to send, the callee expects that we follow
+			 * sendmsg semantics that performs `dup` on it. So we need to clone fd and keep it there.
+			 */
+			if (attached_fd != -1) {
+				rep_elt->attached_fd = dup(attached_fd);
+
+				if (rep_elt->attached_fd == -1) {
+					/*
+					 * We have a problem: file descriptors limit is reached, so we cannot really deal with this
+					 * request
+					 */
+					msg_err_main("cannot duplicate file descriptor to send command to worker %P(%s): %s; failed to send command",
+								 wrk->pid,
+								 g_quark_to_string(wrk->type),
+								 strerror(errno));
+					g_hash_table_unref(rep_elt->pending_elts);
+					g_free(rep_elt);
+				}
+				else {
+					g_hash_table_insert(wrk->control_events_pending, rep_elt, rep_elt);
+					DL_APPEND(res, rep_elt);
+				}
+			}
+			else {
+				g_hash_table_insert(wrk->control_events_pending, rep_elt, rep_elt);
+				DL_APPEND(res, rep_elt);
+			}
 		}
 	}
 
@@ -457,13 +598,13 @@ void rspamd_control_broadcast_srv_cmd(struct rspamd_main *rspamd_main,
 								 rspamd_control_ignore_io_handler, NULL, except_pid);
 }
 
-static gint
+static int
 rspamd_control_finish_handler(struct rspamd_http_connection *conn,
 							  struct rspamd_http_message *msg)
 {
 	struct rspamd_control_session *session = conn->ud;
 	rspamd_ftok_t srch;
-	guint i;
+	unsigned int i;
 	gboolean found = FALSE;
 	struct rspamd_control_reply_elt *cur;
 
@@ -512,7 +653,7 @@ rspamd_control_finish_handler(struct rspamd_http_connection *conn,
 }
 
 void rspamd_control_process_client_socket(struct rspamd_main *rspamd_main,
-										  gint fd, rspamd_inet_addr_t *addr)
+										  int fd, rspamd_inet_addr_t *addr)
 {
 	struct rspamd_control_session *session;
 
@@ -543,8 +684,8 @@ struct rspamd_worker_control_data {
 };
 
 static void
-rspamd_control_default_cmd_handler(gint fd,
-								   gint attached_fd,
+rspamd_control_default_cmd_handler(int fd,
+								   int attached_fd,
 								   struct rspamd_worker_control_data *cd,
 								   struct rspamd_control_command *cmd)
 {
@@ -624,8 +765,8 @@ rspamd_control_default_worker_handler(EV_P_ ev_io *w, int revents)
 	static struct rspamd_control_command cmd;
 	static struct msghdr msg;
 	static struct iovec iov;
-	static guchar fdspace[CMSG_SPACE(sizeof(int))];
-	gint rfd = -1;
+	static unsigned char fdspace[CMSG_SPACE(sizeof(int))];
+	int rfd = -1;
 	gssize r;
 
 	iov.iov_base = &cmd;
@@ -652,16 +793,16 @@ rspamd_control_default_worker_handler(EV_P_ ev_io *w, int revents)
 			close(w->fd);
 		}
 	}
-	else if (r < (gint) sizeof(cmd)) {
-		msg_err("short read of control command: %d of %d", (gint) r,
-				(gint) sizeof(cmd));
+	else if (r < (int) sizeof(cmd)) {
+		msg_err("short read of control command: %d of %d", (int) r,
+				(int) sizeof(cmd));
 
 		if (r == 0) {
 			ev_io_stop(cd->ev_base, &cd->io_ev);
 			close(w->fd);
 		}
 	}
-	else if ((gint) cmd.type >= 0 && cmd.type < RSPAMD_CONTROL_MAX) {
+	else if ((int) cmd.type >= 0 && cmd.type < RSPAMD_CONTROL_MAX) {
 
 		if (msg.msg_controllen >= CMSG_LEN(sizeof(int))) {
 			rfd = *(int *) CMSG_DATA(CMSG_FIRSTHDR(&msg));
@@ -680,7 +821,7 @@ rspamd_control_default_worker_handler(EV_P_ ev_io *w, int revents)
 		}
 	}
 	else {
-		msg_err("unknown command: %d", (gint) cmd.type);
+		msg_err("unknown command: %d", (int) cmd.type);
 	}
 }
 
@@ -723,7 +864,7 @@ void rspamd_control_worker_add_cmd_handler(struct rspamd_worker *worker,
 struct rspamd_srv_reply_data {
 	struct rspamd_worker *worker;
 	struct rspamd_main *srv;
-	gint fd;
+	int fd;
 	struct rspamd_srv_reply rep;
 };
 
@@ -737,7 +878,7 @@ rspamd_control_ignore_io_handler(int fd, short what, void *ud)
 
 	/* At this point we just ignore replies from the workers */
 	if (read(fd, &rep, sizeof(rep)) == -1) {
-		msg_debug("cannot read %d bytes: %s", (int) sizeof(rep), strerror(errno));
+		msg_debug_control("cannot read %d bytes: %s", (int) sizeof(rep), strerror(errno));
 	}
 	rspamd_control_stop_pending(elt);
 }
@@ -838,20 +979,20 @@ rspamd_srv_handler(EV_P_ ev_io *w, int revents)
 {
 	struct rspamd_worker *worker;
 	static struct rspamd_srv_command cmd;
-	struct rspamd_main *srv;
+	struct rspamd_main *rspamd_main;
 	struct rspamd_srv_reply_data *rdata;
 	struct msghdr msg;
 	struct cmsghdr *cmsg;
 	static struct iovec iov;
-	static guchar fdspace[CMSG_SPACE(sizeof(int))];
-	gint *spair, rfd = -1;
-	gchar *nid;
+	static unsigned char fdspace[CMSG_SPACE(sizeof(int))];
+	int *spair, rfd = -1;
+	char *nid;
 	struct rspamd_control_command wcmd;
 	gssize r;
 
 	if (revents == EV_READ) {
 		worker = (struct rspamd_worker *) w->data;
-		srv = worker->srv;
+		rspamd_main = worker->srv;
 		iov.iov_base = &cmd;
 		iov.iov_len = sizeof(cmd);
 		memset(&msg, 0, sizeof(msg));
@@ -864,8 +1005,8 @@ rspamd_srv_handler(EV_P_ ev_io *w, int revents)
 
 		if (r == -1) {
 			if (errno != EAGAIN) {
-				msg_err("cannot read from worker's srv pipe: %s",
-						strerror(errno));
+				msg_err_main("cannot read from worker's srv pipe: %s",
+							 strerror(errno));
 			}
 			else {
 				return;
@@ -876,18 +1017,18 @@ rspamd_srv_handler(EV_P_ ev_io *w, int revents)
 			 * Usually this means that a worker is dead, so do not try to read
 			 * anything
 			 */
-			msg_err("cannot read from worker's srv pipe connection closed; command = %s",
-					rspamd_srv_command_to_string(cmd.type));
+			msg_err_main("cannot read from worker's srv pipe connection closed; command = %s",
+						 rspamd_srv_command_to_string(cmd.type));
 			ev_io_stop(EV_A_ w);
 		}
 		else if (r != sizeof(cmd)) {
-			msg_err("cannot read from worker's srv pipe incomplete command: %d != %d; command = %s",
-					(gint) r, (gint) sizeof(cmd), rspamd_srv_command_to_string(cmd.type));
+			msg_err_main("cannot read from worker's srv pipe incomplete command: %d != %d; command = %s",
+						 (int) r, (int) sizeof(cmd), rspamd_srv_command_to_string(cmd.type));
 		}
 		else {
 			rdata = g_malloc0(sizeof(*rdata));
 			rdata->worker = worker;
-			rdata->srv = srv;
+			rdata->srv = rspamd_main;
 			rdata->rep.id = cmd.id;
 			rdata->rep.type = cmd.type;
 			rdata->fd = -1;
@@ -899,19 +1040,19 @@ rspamd_srv_handler(EV_P_ ev_io *w, int revents)
 
 			switch (cmd.type) {
 			case RSPAMD_SRV_SOCKETPAIR:
-				spair = g_hash_table_lookup(srv->spairs, cmd.cmd.spair.pair_id);
+				spair = g_hash_table_lookup(rspamd_main->spairs, cmd.cmd.spair.pair_id);
 				if (spair == NULL) {
-					spair = g_malloc(sizeof(gint) * 2);
+					spair = g_malloc(sizeof(int) * 2);
 
 					if (rspamd_socketpair(spair, cmd.cmd.spair.af) == -1) {
 						rdata->rep.reply.spair.code = errno;
-						msg_err("cannot create socket pair: %s", strerror(errno));
+						msg_err_main("cannot create socket pair: %s", strerror(errno));
 					}
 					else {
 						nid = g_malloc(sizeof(cmd.cmd.spair.pair_id));
 						memcpy(nid, cmd.cmd.spair.pair_id,
 							   sizeof(cmd.cmd.spair.pair_id));
-						g_hash_table_insert(srv->spairs, nid, spair);
+						g_hash_table_insert(rspamd_main->spairs, nid, spair);
 						rdata->rep.reply.spair.code = 0;
 						rdata->fd = cmd.cmd.spair.pair_num ? spair[1] : spair[0];
 					}
@@ -922,15 +1063,23 @@ rspamd_srv_handler(EV_P_ ev_io *w, int revents)
 				}
 				break;
 			case RSPAMD_SRV_HYPERSCAN_LOADED:
+#ifdef WITH_HYPERSCAN
 				/* Load RE cache to provide it for new forks */
-				if (rspamd_re_cache_is_hs_loaded(srv->cfg->re_cache) != RSPAMD_HYPERSCAN_LOADED_FULL ||
+				if (rspamd_re_cache_is_hs_loaded(rspamd_main->cfg->re_cache) != RSPAMD_HYPERSCAN_LOADED_FULL ||
 					cmd.cmd.hs_loaded.forced) {
 					rspamd_re_cache_load_hyperscan(
-						srv->cfg->re_cache,
+						rspamd_main->cfg->re_cache,
 						cmd.cmd.hs_loaded.cache_dir,
 						false);
 				}
 
+				/* After getting this notice, we can clean up old hyperscan files */
+
+				rspamd_hyperscan_notice_loaded();
+
+				msg_info_main("received hyperscan cache loaded from %s",
+							  cmd.cmd.hs_loaded.cache_dir);
+
 				/* Broadcast command to all workers */
 				memset(&wcmd, 0, sizeof(wcmd));
 				wcmd.type = RSPAMD_CONTROL_HYPERSCAN_LOADED;
@@ -938,8 +1087,9 @@ rspamd_srv_handler(EV_P_ ev_io *w, int revents)
 							   cmd.cmd.hs_loaded.cache_dir,
 							   sizeof(wcmd.cmd.hs_loaded.cache_dir));
 				wcmd.cmd.hs_loaded.forced = cmd.cmd.hs_loaded.forced;
-				rspamd_control_broadcast_cmd(srv, &wcmd, rfd,
+				rspamd_control_broadcast_cmd(rspamd_main, &wcmd, rfd,
 											 rspamd_control_ignore_io_handler, NULL, worker->pid);
+#endif
 				break;
 			case RSPAMD_SRV_MONITORED_CHANGE:
 				/* Broadcast command to all workers */
@@ -950,26 +1100,26 @@ rspamd_srv_handler(EV_P_ ev_io *w, int revents)
 							   sizeof(wcmd.cmd.monitored_change.tag));
 				wcmd.cmd.monitored_change.alive = cmd.cmd.monitored_change.alive;
 				wcmd.cmd.monitored_change.sender = cmd.cmd.monitored_change.sender;
-				rspamd_control_broadcast_cmd(srv, &wcmd, rfd,
+				rspamd_control_broadcast_cmd(rspamd_main, &wcmd, rfd,
 											 rspamd_control_ignore_io_handler, NULL, 0);
 				break;
 			case RSPAMD_SRV_LOG_PIPE:
 				memset(&wcmd, 0, sizeof(wcmd));
 				wcmd.type = RSPAMD_CONTROL_LOG_PIPE;
 				wcmd.cmd.log_pipe.type = cmd.cmd.log_pipe.type;
-				rspamd_control_broadcast_cmd(srv, &wcmd, rfd,
+				rspamd_control_broadcast_cmd(rspamd_main, &wcmd, rfd,
 											 rspamd_control_log_pipe_io_handler, NULL, 0);
 				break;
 			case RSPAMD_SRV_ON_FORK:
 				rdata->rep.reply.on_fork.status = 0;
-				rspamd_control_handle_on_fork(&cmd, srv);
+				rspamd_control_handle_on_fork(&cmd, rspamd_main);
 				break;
 			case RSPAMD_SRV_HEARTBEAT:
 				worker->hb.last_event = ev_time();
 				rdata->rep.reply.heartbeat.status = 0;
 				break;
 			case RSPAMD_SRV_HEALTH:
-				rspamd_fill_health_reply(srv, &rdata->rep);
+				rspamd_fill_health_reply(rspamd_main, &rdata->rep);
 				break;
 			case RSPAMD_SRV_NOTICE_HYPERSCAN_CACHE:
 #ifdef WITH_HYPERSCAN
@@ -984,11 +1134,11 @@ rspamd_srv_handler(EV_P_ ev_io *w, int revents)
 				/* Ensure that memcpy is safe */
 				G_STATIC_ASSERT(sizeof(wcmd.cmd.fuzzy_blocked) == sizeof(cmd.cmd.fuzzy_blocked));
 				memcpy(&wcmd.cmd.fuzzy_blocked, &cmd.cmd.fuzzy_blocked, sizeof(wcmd.cmd.fuzzy_blocked));
-				rspamd_control_broadcast_cmd(srv, &wcmd, rfd,
+				rspamd_control_broadcast_cmd(rspamd_main, &wcmd, rfd,
 											 rspamd_control_ignore_io_handler, NULL, worker->pid);
 				break;
 			default:
-				msg_err("unknown command type: %d", cmd.type);
+				msg_err_main("unknown command type: %d", cmd.type);
 				break;
 			}
 
@@ -1008,7 +1158,7 @@ rspamd_srv_handler(EV_P_ ev_io *w, int revents)
 		rdata = (struct rspamd_srv_reply_data *) w->data;
 		worker = rdata->worker;
 		worker->tmp_data = NULL; /* Avoid race */
-		srv = rdata->srv;
+		rspamd_main = rdata->srv;
 
 		memset(&msg, 0, sizeof(msg));
 
@@ -1032,13 +1182,13 @@ rspamd_srv_handler(EV_P_ ev_io *w, int revents)
 		r = sendmsg(w->fd, &msg, 0);
 
 		if (r == -1) {
-			msg_err("cannot write to worker's srv pipe when writing reply: %s; command = %s",
-					strerror(errno), rspamd_srv_command_to_string(rdata->rep.type));
+			msg_err_main("cannot write to worker's srv pipe when writing reply: %s; command = %s",
+						 strerror(errno), rspamd_srv_command_to_string(rdata->rep.type));
 		}
 		else if (r != sizeof(rdata->rep)) {
-			msg_err("cannot write to worker's srv pipe: %d != %d; command = %s",
-					(int) r, (int) sizeof(rdata->rep),
-					rspamd_srv_command_to_string(rdata->rep.type));
+			msg_err_main("cannot write to worker's srv pipe: %d != %d; command = %s",
+						 (int) r, (int) sizeof(rdata->rep),
+						 rspamd_srv_command_to_string(rdata->rep.type));
 		}
 
 		g_free(rdata);
@@ -1064,7 +1214,7 @@ void rspamd_srv_start_watching(struct rspamd_main *srv,
 struct rspamd_srv_request_data {
 	struct rspamd_worker *worker;
 	struct rspamd_srv_command cmd;
-	gint attached_fd;
+	int attached_fd;
 	struct rspamd_srv_reply rep;
 	rspamd_srv_reply_handler handler;
 	ev_io io_ev;
@@ -1077,10 +1227,10 @@ rspamd_srv_request_handler(EV_P_ ev_io *w, int revents)
 	struct rspamd_srv_request_data *rd = (struct rspamd_srv_request_data *) w->data;
 	struct msghdr msg;
 	struct iovec iov;
-	guchar fdspace[CMSG_SPACE(sizeof(int))];
+	unsigned char fdspace[CMSG_SPACE(sizeof(int))];
 	struct cmsghdr *cmsg;
 	gssize r;
-	gint rfd = -1;
+	int rfd = -1;
 
 	if (revents == EV_WRITE) {
 		/* Send request to server */
@@ -1147,9 +1297,9 @@ rspamd_srv_request_handler(EV_P_ ev_io *w, int revents)
 			goto cleanup;
 		}
 
-		if (r != (gint) sizeof(rd->rep)) {
+		if (r != (int) sizeof(rd->rep)) {
 			msg_err("cannot read from server pipe, invalid length: %d != %d; command = %s",
-					(gint) r, (int) sizeof(rd->rep), rspamd_srv_command_to_string(rd->cmd.type));
+					(int) r, (int) sizeof(rd->rep), rspamd_srv_command_to_string(rd->cmd.type));
 			goto cleanup;
 		}
 
@@ -1176,7 +1326,7 @@ cleanup:
 void rspamd_srv_send_command(struct rspamd_worker *worker,
 							 struct ev_loop *ev_base,
 							 struct rspamd_srv_command *cmd,
-							 gint attached_fd,
+							 int attached_fd,
 							 rspamd_srv_reply_handler handler,
 							 gpointer ud)
 {
@@ -1202,7 +1352,7 @@ void rspamd_srv_send_command(struct rspamd_worker *worker,
 }
 
 enum rspamd_control_type
-rspamd_control_command_from_string(const gchar *str)
+rspamd_control_command_from_string(const char *str)
 {
 	enum rspamd_control_type ret = RSPAMD_CONTROL_MAX;
 
@@ -1244,10 +1394,10 @@ rspamd_control_command_from_string(const gchar *str)
 	return ret;
 }
 
-const gchar *
+const char *
 rspamd_control_command_to_string(enum rspamd_control_type cmd)
 {
-	const gchar *reply = "unknown";
+	const char *reply = "unknown";
 
 	switch (cmd) {
 	case RSPAMD_CONTROL_STAT:
@@ -1287,9 +1437,9 @@ rspamd_control_command_to_string(enum rspamd_control_type cmd)
 	return reply;
 }
 
-const gchar *rspamd_srv_command_to_string(enum rspamd_srv_type cmd)
+const char *rspamd_srv_command_to_string(enum rspamd_srv_type cmd)
 {
-	const gchar *reply = "unknown";
+	const char *reply = "unknown";
 
 	switch (cmd) {
 	case RSPAMD_SRV_SOCKETPAIR:
