@@ -1,11 +1,11 @@
-/*-
- * Copyright 2016 Vsevolod Stakhov
+/*
+ * Copyright 2024 Vsevolod Stakhov
  *
  * Licensed under the Apache License, Version 2.0 (the "License");
  * you may not use this file except in compliance with the License.
  * You may obtain a copy of the License at
  *
- *   http://www.apache.org/licenses/LICENSE-2.0
+ *    http://www.apache.org/licenses/LICENSE-2.0
  *
  * Unless required by applicable law or agreed to in writing, software
  * distributed under the License is distributed on an "AS IS" BASIS,
@@ -21,6 +21,7 @@
 #include "rdns.h"
 #include "cryptobox.h"
 #include "utlist.h"
+#include "contrib/libev/ev.h"
 #include "logger.h"
 #include "contrib/librdns/rdns.h"
 #include "contrib/mumhash/mum.h"
@@ -30,14 +31,14 @@
 
 struct upstream_inet_addr_entry {
 	rspamd_inet_addr_t *addr;
-	guint priority;
+	unsigned int priority;
 	struct upstream_inet_addr_entry *next;
 };
 
 struct upstream_addr_elt {
 	rspamd_inet_addr_t *addr;
-	guint priority;
-	guint errors;
+	unsigned int priority;
+	unsigned int errors;
 };
 
 struct upstream_list_watcher {
@@ -49,17 +50,17 @@ struct upstream_list_watcher {
 };
 
 struct upstream {
-	guint weight;
-	guint cur_weight;
-	guint errors;
-	guint checked;
-	guint dns_requests;
-	gint active_idx;
-	guint ttl;
-	gchar *name;
+	unsigned int weight;
+	unsigned int cur_weight;
+	unsigned int errors;
+	unsigned int checked;
+	unsigned int dns_requests;
+	int active_idx;
+	unsigned int ttl;
+	char *name;
 	ev_timer ev;
-	gdouble last_fail;
-	gdouble last_resolve;
+	double last_fail;
+	double last_resolve;
 	gpointer ud;
 	enum rspamd_upstream_flag flags;
 	struct upstream_list *ls;
@@ -68,12 +69,12 @@ struct upstream {
 
 	struct {
 		GPtrArray *addr; /* struct upstream_addr_elt */
-		guint cur;
+		unsigned int cur;
 	} addrs;
 
 	struct upstream_inet_addr_entry *new_addrs;
 	gpointer data;
-	gchar uid[8];
+	char uid[8];
 	ref_entry_t ref;
 #ifdef UPSTREAMS_THREAD_SAFE
 	rspamd_mutex_t *lock;
@@ -81,25 +82,26 @@ struct upstream {
 };
 
 struct upstream_limits {
-	gdouble revive_time;
-	gdouble revive_jitter;
-	gdouble error_time;
-	gdouble dns_timeout;
-	gdouble lazy_resolve_time;
-	guint max_errors;
-	guint dns_retransmits;
+	double revive_time;
+	double revive_jitter;
+	double error_time;
+	double dns_timeout;
+	double lazy_resolve_time;
+	double resolve_min_interval;
+	unsigned int max_errors;
+	unsigned int dns_retransmits;
 };
 
 struct upstream_list {
-	gchar *ups_line;
+	char *ups_line;
 	struct upstream_ctx *ctx;
 	GPtrArray *ups;
 	GPtrArray *alive;
 	struct upstream_list_watcher *watchers;
-	guint64 hash_seed;
+	uint64_t hash_seed;
 	const struct upstream_limits *limits;
 	enum rspamd_upstream_flag flags;
-	guint cur_elt;
+	unsigned int cur_elt;
 	enum rspamd_upstream_rotation rot_alg;
 #ifdef UPSTREAMS_THREAD_SAFE
 	rspamd_mutex_t *lock;
@@ -145,20 +147,21 @@ INIT_LOG_MODULE(upstream)
 
 /* 4 errors in 10 seconds */
 #define DEFAULT_MAX_ERRORS 4
-static const guint default_max_errors = DEFAULT_MAX_ERRORS;
+static const unsigned int default_max_errors = DEFAULT_MAX_ERRORS;
 #define DEFAULT_REVIVE_TIME 60
-static const gdouble default_revive_time = DEFAULT_REVIVE_TIME;
+static const double default_revive_time = DEFAULT_REVIVE_TIME;
 #define DEFAULT_REVIVE_JITTER 0.4
-static const gdouble default_revive_jitter = DEFAULT_REVIVE_JITTER;
+static const double default_revive_jitter = DEFAULT_REVIVE_JITTER;
 #define DEFAULT_ERROR_TIME 10
-static const gdouble default_error_time = DEFAULT_ERROR_TIME;
+static const double default_error_time = DEFAULT_ERROR_TIME;
 #define DEFAULT_DNS_TIMEOUT 1.0
-static const gdouble default_dns_timeout = DEFAULT_DNS_TIMEOUT;
+static const double default_dns_timeout = DEFAULT_DNS_TIMEOUT;
 #define DEFAULT_DNS_RETRANSMITS 2
-static const guint default_dns_retransmits = DEFAULT_DNS_RETRANSMITS;
-/* TODO: make it configurable */
+static const unsigned int default_dns_retransmits = DEFAULT_DNS_RETRANSMITS;
 #define DEFAULT_LAZY_RESOLVE_TIME 3600.0
-static const gdouble default_lazy_resolve_time = DEFAULT_LAZY_RESOLVE_TIME;
+static const double default_lazy_resolve_time = DEFAULT_LAZY_RESOLVE_TIME;
+#define DEFAULT_RESOLVE_MIN_INTERVAL 60.0
+static const double default_resolve_min_interval = DEFAULT_RESOLVE_MIN_INTERVAL;
 
 static const struct upstream_limits default_limits = {
 	.revive_time = DEFAULT_REVIVE_TIME,
@@ -168,6 +171,7 @@ static const struct upstream_limits default_limits = {
 	.dns_retransmits = DEFAULT_DNS_RETRANSMITS,
 	.max_errors = DEFAULT_MAX_ERRORS,
 	.lazy_resolve_time = DEFAULT_LAZY_RESOLVE_TIME,
+	.resolve_min_interval = DEFAULT_RESOLVE_MIN_INTERVAL,
 };
 
 static void rspamd_upstream_lazy_resolve_cb(struct ev_loop *, ev_timer *, int);
@@ -198,6 +202,15 @@ void rspamd_upstreams_library_config(struct rspamd_config *cfg,
 	if (cfg->dns_timeout) {
 		ctx->limits.dns_timeout = cfg->dns_timeout;
 	}
+	if (cfg->upstream_resolve_min_interval) {
+		ctx->limits.resolve_min_interval = cfg->upstream_resolve_min_interval;
+	}
+
+	/* Some sanity checks */
+	if (ctx->limits.resolve_min_interval > ctx->limits.revive_time) {
+		/* We must be able to resolve host during the revive time */
+		ctx->limits.resolve_min_interval = ctx->limits.revive_time;
+	}
 
 	ctx->event_loop = event_loop;
 	ctx->res = resolver;
@@ -214,7 +227,7 @@ void rspamd_upstreams_library_config(struct rspamd_config *cfg,
 			upstream = cur->data;
 			if (!ev_can_stop(&upstream->ev) && upstream->ls &&
 				!(upstream->flags & RSPAMD_UPSTREAM_FLAG_NORESOLVE)) {
-				gdouble when;
+				double when;
 
 				if (upstream->flags & RSPAMD_UPSTREAM_FLAG_SRV_RESOLVE) {
 					/* Resolve them immediately ! */
@@ -277,7 +290,7 @@ rspamd_upstreams_library_init(void)
 	return ctx;
 }
 
-static gint
+static int
 rspamd_upstream_af_to_weight(const rspamd_inet_addr_t *addr)
 {
 	int ret;
@@ -300,12 +313,12 @@ rspamd_upstream_af_to_weight(const rspamd_inet_addr_t *addr)
 /*
  * Select IPv4 addresses before IPv6
  */
-static gint
+static int
 rspamd_upstream_addr_sort_func(gconstpointer a, gconstpointer b)
 {
 	const struct upstream_addr_elt *ip1 = *(const struct upstream_addr_elt **) a,
 								   *ip2 = *(const struct upstream_addr_elt **) b;
-	gint w1, w2;
+	int w1, w2;
 
 	if (ip1->priority == 0 && ip2->priority == 0) {
 		w1 = rspamd_upstream_af_to_weight(ip1->addr);
@@ -335,7 +348,7 @@ rspamd_upstream_set_active(struct upstream_list *ls, struct upstream *upstream)
 		}
 
 		/* Start lazy (or not so lazy) names resolution */
-		gdouble when;
+		double when;
 
 		if (upstream->flags & RSPAMD_UPSTREAM_FLAG_SRV_RESOLVE) {
 			/* Resolve them immediately ! */
@@ -370,7 +383,7 @@ rspamd_upstream_addr_elt_dtor(gpointer a)
 static void
 rspamd_upstream_update_addrs(struct upstream *upstream)
 {
-	guint addr_cnt, i, port;
+	unsigned int addr_cnt, i, port;
 	gboolean seen_addr, reset_errors = FALSE;
 	struct upstream_inet_addr_entry *cur, *tmp;
 	GPtrArray *new_addrs;
@@ -499,9 +512,9 @@ rspamd_upstream_dns_cb(struct rdns_reply *reply, void *arg)
 
 struct rspamd_upstream_srv_dns_cb {
 	struct upstream *up;
-	guint priority;
-	guint port;
-	guint requests_inflight;
+	unsigned int priority;
+	unsigned int port;
+	unsigned int requests_inflight;
 };
 
 /* Used when we have resolved SRV record and resolved addrs */
@@ -640,21 +653,19 @@ static void
 rspamd_upstream_resolve_addrs(const struct upstream_list *ls,
 							  struct upstream *upstream)
 {
-	/* XXX: maybe make it configurable */
-	static const gdouble min_resolve_interval = 60.0;
 
 	if (upstream->ctx->res != NULL &&
 		upstream->ctx->configured &&
 		upstream->dns_requests == 0 &&
 		!(upstream->flags & RSPAMD_UPSTREAM_FLAG_NORESOLVE)) {
 
-		gdouble now = ev_now(upstream->ctx->event_loop);
+		double now = ev_now(upstream->ctx->event_loop);
 
-		if (now - upstream->last_resolve < min_resolve_interval) {
+		if (now - upstream->last_resolve < upstream->ctx->limits.resolve_min_interval) {
 			msg_info_upstream("do not resolve upstream %s as it was checked %.0f "
 							  "seconds ago (%.0f is minimum)",
 							  upstream->name, now - upstream->last_resolve,
-							  min_resolve_interval);
+							  upstream->ctx->limits.resolve_min_interval);
 
 			return;
 		}
@@ -750,11 +761,12 @@ rspamd_upstream_lazy_resolve_cb(struct ev_loop *loop, ev_timer *w, int revents)
 static void
 rspamd_upstream_set_inactive(struct upstream_list *ls, struct upstream *upstream)
 {
-	gdouble ntim;
-	guint i;
+	double ntim;
+	unsigned int i;
 	struct upstream *cur;
 	struct upstream_list_watcher *w;
 
+	g_assert(upstream != NULL);
 	RSPAMD_UPSTREAM_LOCK(ls);
 	g_ptr_array_remove_index(ls->alive, upstream->active_idx);
 	upstream->active_idx = -1;
@@ -798,13 +810,14 @@ rspamd_upstream_set_inactive(struct upstream_list *ls, struct upstream *upstream
 
 void rspamd_upstream_fail(struct upstream *upstream,
 						  gboolean addr_failure,
-						  const gchar *reason)
+						  const char *reason)
 {
-	gdouble error_rate = 0, max_error_rate = 0;
-	gdouble sec_last, sec_cur;
+	double error_rate = 0, max_error_rate = 0;
+	double sec_last, sec_cur;
 	struct upstream_addr_elt *addr_elt;
 	struct upstream_list_watcher *w;
 
+	g_assert(upstream != NULL);
 	msg_debug_upstream("upstream %s failed; reason: %s",
 					   upstream->name,
 					   reason);
@@ -846,8 +859,8 @@ void rspamd_upstream_fail(struct upstream *upstream,
 				}
 
 				if (sec_cur - sec_last >= upstream->ls->limits->error_time) {
-					error_rate = ((gdouble) upstream->errors) / (sec_cur - sec_last);
-					max_error_rate = ((gdouble) upstream->ls->limits->max_errors) /
+					error_rate = ((double) upstream->errors) / (sec_cur - sec_last);
+					max_error_rate = ((double) upstream->ls->limits->max_errors) /
 									 upstream->ls->limits->error_time;
 				}
 
@@ -941,7 +954,7 @@ void rspamd_upstream_ok(struct upstream *upstream)
 	RSPAMD_UPSTREAM_UNLOCK(upstream);
 }
 
-void rspamd_upstream_set_weight(struct upstream *up, guint weight)
+void rspamd_upstream_set_weight(struct upstream *up, unsigned int weight)
 {
 	RSPAMD_UPSTREAM_LOCK(up);
 	up->weight = weight;
@@ -1025,7 +1038,7 @@ rspamd_upstream_dtor(struct upstream *up)
 rspamd_inet_addr_t *
 rspamd_upstream_addr_next(struct upstream *up)
 {
-	guint idx, next_idx;
+	unsigned int idx, next_idx;
 	struct upstream_addr_elt *e1, *e2;
 
 	do {
@@ -1049,13 +1062,13 @@ rspamd_upstream_addr_cur(const struct upstream *up)
 	return elt->addr;
 }
 
-const gchar *
+const char *
 rspamd_upstream_name(struct upstream *up)
 {
 	return up->name;
 }
 
-gint rspamd_upstream_port(struct upstream *up)
+int rspamd_upstream_port(struct upstream *up)
 {
 	struct upstream_addr_elt *elt;
 
@@ -1064,13 +1077,13 @@ gint rspamd_upstream_port(struct upstream *up)
 }
 
 gboolean
-rspamd_upstreams_add_upstream(struct upstream_list *ups, const gchar *str,
-							  guint16 def_port, enum rspamd_upstream_parse_type parse_type,
+rspamd_upstreams_add_upstream(struct upstream_list *ups, const char *str,
+							  uint16_t def_port, enum rspamd_upstream_parse_type parse_type,
 							  void *data)
 {
 	struct upstream *upstream;
 	GPtrArray *addrs = NULL;
-	guint i, slen;
+	unsigned int i, slen;
 	rspamd_inet_addr_t *addr;
 	enum rspamd_parse_host_port_result ret = RSPAMD_PARSE_ADDR_FAIL;
 
@@ -1081,7 +1094,7 @@ rspamd_upstreams_add_upstream(struct upstream_list *ups, const gchar *str,
 	case RSPAMD_UPSTREAM_PARSE_DEFAULT:
 		if (slen > sizeof("service=") &&
 			RSPAMD_LEN_CHECK_STARTS_WITH(str, slen, "service=")) {
-			const gchar *plus_pos, *service_pos, *semicolon_pos;
+			const char *plus_pos, *service_pos, *semicolon_pos;
 
 			/* Accept service=srv_name+hostname[:priority] */
 			service_pos = str + sizeof("service=") - 1;
@@ -1102,17 +1115,17 @@ rspamd_upstreams_add_upstream(struct upstream_list *ups, const gchar *str,
 				 * where <domain> is string between semicolon_pos and plus_pos +1
 				 * while service is a string between service_pos and plus_pos
 				 */
-				guint namelen = (semicolon_pos - (plus_pos + 1)) +
-								(plus_pos - service_pos) +
-								(sizeof("tcp") - 1) +
-								4;
+				unsigned int namelen = (semicolon_pos - (plus_pos + 1)) +
+									   (plus_pos - service_pos) +
+									   (sizeof("tcp") - 1) +
+									   4;
 				addrs = g_ptr_array_sized_new(1);
 				upstream->name = ups->ctx ? rspamd_mempool_alloc(ups->ctx->pool, namelen + 1) : g_malloc(namelen + 1);
 
 				rspamd_snprintf(upstream->name, namelen + 1,
 								"_%*s._tcp.%*s",
-								(gint) (plus_pos - service_pos), service_pos,
-								(gint) (semicolon_pos - (plus_pos + 1)), plus_pos + 1);
+								(int) (plus_pos - service_pos), service_pos,
+								(int) (semicolon_pos - (plus_pos + 1)), plus_pos + 1);
 				upstream->flags |= RSPAMD_UPSTREAM_FLAG_SRV_RESOLVE;
 				ret = RSPAMD_PARSE_ADDR_RESOLVED;
 			}
@@ -1199,10 +1212,10 @@ rspamd_upstreams_add_upstream(struct upstream_list *ups, const gchar *str,
 		upstream->ctx_pos = g_queue_peek_tail_link(ups->ctx->upstreams);
 	}
 
-	guint h = rspamd_cryptobox_fast_hash(upstream->name,
-										 strlen(upstream->name), 0);
+	unsigned int h = rspamd_cryptobox_fast_hash(upstream->name,
+												strlen(upstream->name), 0);
 	memset(upstream->uid, 0, sizeof(upstream->uid));
-	rspamd_encode_base32_buf((const guchar *) &h, sizeof(h),
+	rspamd_encode_base32_buf((const unsigned char *) &h, sizeof(h),
 							 upstream->uid, sizeof(upstream->uid) - 1, RSPAMD_BASE32_DEFAULT);
 
 	msg_debug_upstream("added upstream %s (%s)", upstream->name,
@@ -1246,12 +1259,12 @@ rspamd_upstream_add_addr(struct upstream *up, rspamd_inet_addr_t *addr)
 
 gboolean
 rspamd_upstreams_parse_line_len(struct upstream_list *ups,
-								const gchar *str, gsize len, guint16 def_port, void *data)
+								const char *str, gsize len, uint16_t def_port, void *data)
 {
-	const gchar *end = str + len, *p = str;
-	const gchar *separators = ";, \n\r\t";
-	gchar *tmp;
-	guint span_len;
+	const char *end = str + len, *p = str;
+	const char *separators = ";, \n\r\t";
+	char *tmp;
+	unsigned int span_len;
 	gboolean ret = FALSE;
 
 	if (RSPAMD_LEN_CHECK_STARTS_WITH(p, len, "random:")) {
@@ -1305,7 +1318,7 @@ rspamd_upstreams_parse_line_len(struct upstream_list *ups,
 
 gboolean
 rspamd_upstreams_parse_line(struct upstream_list *ups,
-							const gchar *str, guint16 def_port, void *data)
+							const char *str, uint16_t def_port, void *data)
 {
 	return rspamd_upstreams_parse_line_len(ups, str, strlen(str),
 										   def_port, data);
@@ -1313,7 +1326,7 @@ rspamd_upstreams_parse_line(struct upstream_list *ups,
 
 gboolean
 rspamd_upstreams_from_ucl(struct upstream_list *ups,
-						  const ucl_object_t *in, guint16 def_port, void *data)
+						  const ucl_object_t *in, uint16_t def_port, void *data)
 {
 	gboolean ret = FALSE;
 	const ucl_object_t *cur;
@@ -1335,7 +1348,7 @@ rspamd_upstreams_from_ucl(struct upstream_list *ups,
 
 void rspamd_upstreams_destroy(struct upstream_list *ups)
 {
-	guint i;
+	unsigned int i;
 	struct upstream *up;
 	struct upstream_list_watcher *w, *tmp;
 
@@ -1400,7 +1413,7 @@ rspamd_upstream_get_random(struct upstream_list *ups,
 						   struct upstream *except)
 {
 	for (;;) {
-		guint idx = ottery_rand_range(ups->alive->len - 1);
+		unsigned int idx = ottery_rand_range(ups->alive->len - 1);
 		struct upstream *up;
 
 		up = g_ptr_array_index(ups->alive, idx);
@@ -1418,9 +1431,9 @@ rspamd_upstream_get_round_robin(struct upstream_list *ups,
 								struct upstream *except,
 								gboolean use_cur)
 {
-	guint max_weight = 0, min_checked = G_MAXUINT;
+	unsigned int max_weight = 0, min_checked = G_MAXUINT;
 	struct upstream *up = NULL, *selected = NULL, *min_checked_sel = NULL;
-	guint i;
+	unsigned int i;
 
 	/* Select upstream with the maximum cur_weight */
 	RSPAMD_UPSTREAM_LOCK(ups);
@@ -1491,10 +1504,10 @@ rspamd_upstream_get_round_robin(struct upstream_list *ups,
  *
  * http://arxiv.org/abs/1406.2294
  */
-static guint32
-rspamd_consistent_hash(guint64 key, guint32 nbuckets)
+static uint32_t
+rspamd_consistent_hash(uint64_t key, uint32_t nbuckets)
 {
-	gint64 b = -1, j = 0;
+	int64_t b = -1, j = 0;
 
 	while (j < nbuckets) {
 		b = j;
@@ -1508,11 +1521,11 @@ rspamd_consistent_hash(guint64 key, guint32 nbuckets)
 static struct upstream *
 rspamd_upstream_get_hashed(struct upstream_list *ups,
 						   struct upstream *except,
-						   const guint8 *key, guint keylen)
+						   const uint8_t *key, unsigned int keylen)
 {
-	guint64 k;
-	guint32 idx;
-	static const guint max_tries = 20;
+	uint64_t k;
+	uint32_t idx;
+	static const unsigned int max_tries = 20;
 	struct upstream *up = NULL;
 
 	/* Generate 64 bits input key */
@@ -1523,7 +1536,7 @@ rspamd_upstream_get_hashed(struct upstream_list *ups,
 	/*
 	 * Select new upstream from all upstreams
 	 */
-	for (guint i = 0; i < max_tries; i++) {
+	for (unsigned int i = 0; i < max_tries; i++) {
 		idx = rspamd_consistent_hash(k, ups->ups->len);
 		up = g_ptr_array_index(ups->ups, idx);
 
@@ -1553,7 +1566,7 @@ static struct upstream *
 rspamd_upstream_get_common(struct upstream_list *ups,
 						   struct upstream *except,
 						   enum rspamd_upstream_rotation default_type,
-						   const guchar *key, gsize keylen,
+						   const unsigned char *key, gsize keylen,
 						   gboolean forced)
 {
 	enum rspamd_upstream_rotation type;
@@ -1621,7 +1634,7 @@ end:
 struct upstream *
 rspamd_upstream_get(struct upstream_list *ups,
 					enum rspamd_upstream_rotation default_type,
-					const guchar *key, gsize keylen)
+					const unsigned char *key, gsize keylen)
 {
 	return rspamd_upstream_get_common(ups, NULL, default_type, key, keylen, FALSE);
 }
@@ -1629,7 +1642,7 @@ rspamd_upstream_get(struct upstream_list *ups,
 struct upstream *
 rspamd_upstream_get_forced(struct upstream_list *ups,
 						   enum rspamd_upstream_rotation forced_type,
-						   const guchar *key, gsize keylen)
+						   const unsigned char *key, gsize keylen)
 {
 	return rspamd_upstream_get_common(ups, NULL, forced_type, key, keylen, TRUE);
 }
@@ -1637,7 +1650,7 @@ rspamd_upstream_get_forced(struct upstream_list *ups,
 struct upstream *rspamd_upstream_get_except(struct upstream_list *ups,
 											struct upstream *except,
 											enum rspamd_upstream_rotation default_type,
-											const guchar *key, gsize keylen)
+											const unsigned char *key, gsize keylen)
 {
 	return rspamd_upstream_get_common(ups, except, default_type, key, keylen, FALSE);
 }
@@ -1651,6 +1664,7 @@ void rspamd_upstream_reresolve(struct upstream_ctx *ctx)
 
 	while (cur) {
 		up = cur->data;
+		g_assert(up != NULL);
 		REF_RETAIN(up);
 		rspamd_upstream_resolve_addrs(up->ls, up);
 		REF_RELEASE(up);
@@ -1678,7 +1692,7 @@ void rspamd_upstreams_foreach(struct upstream_list *ups,
 							  rspamd_upstream_traverse_func cb, void *ud)
 {
 	struct upstream *up;
-	guint i;
+	unsigned int i;
 
 	for (i = 0; i < ups->ups->len; i++) {
 		up = g_ptr_array_index(ups->ups, i);
@@ -1688,12 +1702,12 @@ void rspamd_upstreams_foreach(struct upstream_list *ups,
 }
 
 void rspamd_upstreams_set_limits(struct upstream_list *ups,
-								 gdouble revive_time,
-								 gdouble revive_jitter,
-								 gdouble error_time,
-								 gdouble dns_timeout,
-								 guint max_errors,
-								 guint dns_retransmits)
+								 double revive_time,
+								 double revive_jitter,
+								 double error_time,
+								 double dns_timeout,
+								 unsigned int max_errors,
+								 unsigned int dns_retransmits)
 {
 	struct upstream_limits *nlimits;
 	g_assert(ups != NULL);
