@@ -1,11 +1,11 @@
-/*-
- * Copyright 2016 Vsevolod Stakhov
+/*
+ * Copyright 2024 Vsevolod Stakhov
  *
  * Licensed under the Apache License, Version 2.0 (the "License");
  * you may not use this file except in compliance with the License.
  * You may obtain a copy of the License at
  *
- *   http://www.apache.org/licenses/LICENSE-2.0
+ *    http://www.apache.org/licenses/LICENSE-2.0
  *
  * Unless required by applicable law or agreed to in writing, software
  * distributed under the License is distributed on an "AS IS" BASIS,
@@ -760,34 +760,34 @@ static const struct luaL_reg ev_baselib_m[] = {
 	{"__tostring", rspamd_lua_class_tostring},
 	{NULL, NULL}};
 
-static gint64
-lua_check_int64(lua_State *L, gint pos)
+static int64_t
+lua_check_int64(lua_State *L, int pos)
 {
-	void *ud = rspamd_lua_check_udata(L, pos, "rspamd{int64}");
+	void *ud = rspamd_lua_check_udata(L, pos, rspamd_int64_classname);
 	luaL_argcheck(L, ud != NULL, pos, "'int64' expected");
-	return ud ? *((gint64 *) ud) : 0LL;
+	return ud ? *((int64_t *) ud) : 0LL;
 }
 
 
-static gint
+static int
 lua_util_create_event_base(lua_State *L)
 {
 	LUA_TRACE_POINT;
 	struct ev_loop **pev_base;
 
 	pev_base = lua_newuserdata(L, sizeof(struct ev_loop *));
-	rspamd_lua_setclass(L, "rspamd{ev_base}", -1);
+	rspamd_lua_setclass(L, rspamd_ev_base_classname, -1);
 	*pev_base = ev_loop_new(EVFLAG_SIGNALFD | EVBACKEND_ALL);
 
 	return 1;
 }
 
-static gint
+static int
 lua_util_load_rspamd_config(lua_State *L)
 {
 	LUA_TRACE_POINT;
 	struct rspamd_config *cfg, **pcfg;
-	const gchar *cfg_name;
+	const char *cfg_name;
 
 	cfg_name = luaL_checkstring(L, 1);
 
@@ -802,7 +802,7 @@ lua_util_load_rspamd_config(lua_State *L)
 		else {
 			rspamd_config_post_load(cfg, 0);
 			pcfg = lua_newuserdata(L, sizeof(struct rspamd_config *));
-			rspamd_lua_setclass(L, "rspamd{config}", -1);
+			rspamd_lua_setclass(L, rspamd_config_classname, -1);
 			*pcfg = cfg;
 		}
 	}
@@ -810,13 +810,13 @@ lua_util_load_rspamd_config(lua_State *L)
 	return 1;
 }
 
-static gint
+static int
 parse_config_options(const char *str_options)
 {
-	gint ret = 0;
-	gchar **vec;
-	const gchar *str;
-	guint i, l;
+	int ret = 0;
+	char **vec;
+	const char *str;
+	unsigned int i, l;
 
 	vec = g_strsplit_set(str_options, ",;", -1);
 	if (vec) {
@@ -853,16 +853,16 @@ parse_config_options(const char *str_options)
 	return ret;
 }
 
-static gint
+static int
 lua_util_config_from_ucl(lua_State *L)
 {
 	LUA_TRACE_POINT;
 	struct rspamd_config *cfg = NULL, **pcfg;
-	struct rspamd_rcl_section *top;
+	struct rspamd_rcl_sections_map *top;
 	GError *err = NULL;
 	ucl_object_t *obj;
 	const char *str_options = NULL;
-	gint int_options = 0;
+	int int_options = 0;
 
 
 	obj = ucl_object_lua_import(L, 1);
@@ -882,10 +882,10 @@ lua_util_config_from_ucl(lua_State *L)
 		cfg = rspamd_config_new(RSPAMD_CONFIG_INIT_SKIP_LUA);
 		cfg->lua_state = L;
 
-		cfg->rcl_obj = obj;
+		cfg->cfg_ucl_obj = obj;
 		top = rspamd_rcl_config_init(cfg, NULL);
 
-		if (!rspamd_rcl_parse(top, cfg, cfg, cfg->cfg_pool, cfg->rcl_obj, &err)) {
+		if (!rspamd_rcl_parse(top, cfg, cfg, cfg->cfg_pool, cfg->cfg_ucl_obj, &err)) {
 			msg_err("rcl parse error: %s", err->message);
 			ucl_object_unref(obj);
 			lua_pushnil(L);
@@ -898,9 +898,11 @@ lua_util_config_from_ucl(lua_State *L)
 
 			rspamd_config_post_load(cfg, int_options);
 			pcfg = lua_newuserdata(L, sizeof(struct rspamd_config *));
-			rspamd_lua_setclass(L, "rspamd{config}", -1);
+			rspamd_lua_setclass(L, rspamd_config_classname, -1);
 			*pcfg = cfg;
 		}
+
+		rspamd_rcl_sections_free(top);
 	}
 
 	return 1;
@@ -917,12 +919,12 @@ lua_util_task_fin(struct rspamd_task *task, void *ud)
 	return TRUE;
 }
 
-static gint
+static int
 lua_util_process_message(lua_State *L)
 {
 	LUA_TRACE_POINT;
 	struct rspamd_config *cfg = lua_check_config(L, 1);
-	const gchar *message;
+	const char *message;
 	gsize mlen;
 	struct rspamd_task *task;
 	struct ev_loop *base;
@@ -977,48 +979,36 @@ lua_util_process_message(lua_State *L)
 	return 1;
 }
 
-static gint
+static int
 lua_util_encode_base64(lua_State *L)
 {
 	LUA_TRACE_POINT;
 	struct rspamd_lua_text *t;
-	const gchar *s = NULL;
-	gchar *out;
-	gsize inlen, outlen;
-	guint str_lim = 0;
+	char *out;
+	gsize outlen;
+	long str_lim = 0;
 	gboolean fold = FALSE;
 
-	if (lua_type(L, 1) == LUA_TSTRING) {
-		s = luaL_checklstring(L, 1, &inlen);
-	}
-	else if (lua_type(L, 1) == LUA_TUSERDATA) {
-		t = lua_check_text(L, 1);
-
-		if (t != NULL) {
-			s = t->start;
-			inlen = t->len;
-		}
-	}
+	t = lua_check_text_or_string(L, 1);
 
 	if (lua_gettop(L) > 1) {
-		str_lim = luaL_checknumber(L, 2);
-
-		fold = !!(str_lim > 0);
+		str_lim = luaL_checkinteger(L, 2);
+		fold = str_lim > 0;
 	}
 
-	if (s == NULL) {
-		lua_pushnil(L);
+	if (t == NULL) {
+		return luaL_error(L, "invalid arguments");
 	}
 	else {
 
 		if (fold) {
-			out = rspamd_encode_base64(s, inlen, str_lim, &outlen);
+			out = rspamd_encode_base64(t->start, t->len, str_lim, &outlen);
 		}
 		else {
 			enum rspamd_newlines_type how = RSPAMD_TASK_NEWLINES_CRLF;
 
 			if (lua_type(L, 3) == LUA_TSTRING) {
-				const gchar *how_str = lua_tostring(L, 3);
+				const char *how_str = lua_tostring(L, 3);
 
 				if (g_ascii_strcasecmp(how_str, "cr") == 0) {
 					how = RSPAMD_TASK_NEWLINES_CR;
@@ -1031,16 +1021,11 @@ lua_util_encode_base64(lua_State *L)
 				}
 			}
 
-			out = rspamd_encode_base64_fold(s, inlen, str_lim, &outlen, how);
+			out = rspamd_encode_base64_fold(t->start, t->len, str_lim, &outlen, how);
 		}
 
 		if (out != NULL) {
-			t = lua_newuserdata(L, sizeof(*t));
-			rspamd_lua_setclass(L, "rspamd{text}", -1);
-			t->start = out;
-			t->len = outlen;
-			/* Need destruction */
-			t->flags = RSPAMD_TEXT_FLAG_OWN;
+			lua_new_text(L, out, outlen, TRUE);
 		}
 		else {
 			lua_pushnil(L);
@@ -1050,15 +1035,15 @@ lua_util_encode_base64(lua_State *L)
 	return 1;
 }
 
-static gint
+static int
 lua_util_encode_qp(lua_State *L)
 {
 	LUA_TRACE_POINT;
 	struct rspamd_lua_text *t;
-	const gchar *s = NULL;
-	gchar *out;
+	const char *s = NULL;
+	char *out;
 	gsize inlen, outlen;
-	guint str_lim = 0;
+	unsigned int str_lim = 0;
 
 	if (lua_type(L, 1) == LUA_TSTRING) {
 		s = luaL_checklstring(L, 1, &inlen);
@@ -1083,7 +1068,7 @@ lua_util_encode_qp(lua_State *L)
 		enum rspamd_newlines_type how = RSPAMD_TASK_NEWLINES_CRLF;
 
 		if (lua_type(L, 3) == LUA_TSTRING) {
-			const gchar *how_str = lua_tostring(L, 3);
+			const char *how_str = lua_tostring(L, 3);
 
 			if (g_ascii_strcasecmp(how_str, "cr") == 0) {
 				how = RSPAMD_TASK_NEWLINES_CR;
@@ -1100,7 +1085,7 @@ lua_util_encode_qp(lua_State *L)
 
 		if (out != NULL) {
 			t = lua_newuserdata(L, sizeof(*t));
-			rspamd_lua_setclass(L, "rspamd{text}", -1);
+			rspamd_lua_setclass(L, rspamd_text_classname, -1);
 			t->start = out;
 			t->len = outlen;
 			/* Need destruction */
@@ -1114,12 +1099,12 @@ lua_util_encode_qp(lua_State *L)
 	return 1;
 }
 
-static gint
+static int
 lua_util_decode_qp(lua_State *L)
 {
 	LUA_TRACE_POINT;
 	struct rspamd_lua_text *t, *out;
-	const gchar *s = NULL;
+	const char *s = NULL;
 	gsize inlen = 0;
 	gssize outlen;
 
@@ -1140,10 +1125,10 @@ lua_util_decode_qp(lua_State *L)
 	}
 	else {
 		out = lua_newuserdata(L, sizeof(*t));
-		rspamd_lua_setclass(L, "rspamd{text}", -1);
+		rspamd_lua_setclass(L, rspamd_text_classname, -1);
 		out->start = g_malloc(inlen + 1);
 		out->flags = RSPAMD_TEXT_FLAG_OWN;
-		outlen = rspamd_decode_qp_buf(s, inlen, (gchar *) out->start, inlen + 1);
+		outlen = rspamd_decode_qp_buf(s, inlen, (char *) out->start, inlen + 1);
 
 		if (outlen > 0) {
 			out->len = outlen;
@@ -1160,12 +1145,12 @@ lua_util_decode_qp(lua_State *L)
 	return 1;
 }
 
-static gint
+static int
 lua_util_decode_base64(lua_State *L)
 {
 	LUA_TRACE_POINT;
 	struct rspamd_lua_text *t;
-	const gchar *s = NULL;
+	const char *s = NULL;
 	gsize inlen = 0, outlen;
 
 	if (lua_type(L, 1) == LUA_TSTRING) {
@@ -1182,11 +1167,11 @@ lua_util_decode_base64(lua_State *L)
 
 	if (s != NULL) {
 		t = lua_newuserdata(L, sizeof(*t));
-		rspamd_lua_setclass(L, "rspamd{text}", -1);
+		rspamd_lua_setclass(L, rspamd_text_classname, -1);
 		t->len = (inlen / 4) * 3 + 3;
 		t->start = g_malloc(t->len);
 
-		rspamd_cryptobox_base64_decode(s, inlen, (guchar *) t->start,
+		rspamd_cryptobox_base64_decode(s, inlen, (unsigned char *) t->start,
 									   &outlen);
 		t->len = outlen;
 		t->flags = RSPAMD_TEXT_FLAG_OWN;
@@ -1198,13 +1183,13 @@ lua_util_decode_base64(lua_State *L)
 	return 1;
 }
 
-static gint
+static int
 lua_util_encode_base32(lua_State *L)
 {
 	LUA_TRACE_POINT;
 	struct rspamd_lua_text *t;
-	const gchar *s = NULL;
-	gchar *out;
+	const char *s = NULL;
+	char *out;
 	enum rspamd_base32_type btype = RSPAMD_BASE32_DEFAULT;
 	gsize inlen, outlen;
 
@@ -1237,7 +1222,7 @@ lua_util_encode_base32(lua_State *L)
 		if (out != NULL) {
 			t = lua_newuserdata(L, sizeof(*t));
 			outlen = strlen(out);
-			rspamd_lua_setclass(L, "rspamd{text}", -1);
+			rspamd_lua_setclass(L, rspamd_text_classname, -1);
 			t->start = out;
 			t->len = outlen;
 			/* Need destruction */
@@ -1251,12 +1236,12 @@ lua_util_encode_base32(lua_State *L)
 	return 1;
 }
 
-static gint
+static int
 lua_util_decode_base32(lua_State *L)
 {
 	LUA_TRACE_POINT;
 	struct rspamd_lua_text *t;
-	const gchar *s = NULL;
+	const char *s = NULL;
 	gsize inlen, outlen;
 	enum rspamd_base32_type btype = RSPAMD_BASE32_DEFAULT;
 
@@ -1281,14 +1266,14 @@ lua_util_decode_base32(lua_State *L)
 	}
 
 	if (s != NULL) {
-		guchar *decoded;
+		unsigned char *decoded;
 
 		decoded = rspamd_decode_base32(s, inlen, &outlen, btype);
 
 		if (decoded) {
 			t = lua_newuserdata(L, sizeof(*t));
-			rspamd_lua_setclass(L, "rspamd{text}", -1);
-			t->start = (const gchar *) decoded;
+			rspamd_lua_setclass(L, rspamd_text_classname, -1);
+			t->start = (const char *) decoded;
 			t->len = outlen;
 			t->flags = RSPAMD_TEXT_FLAG_OWN;
 		}
@@ -1303,7 +1288,7 @@ lua_util_decode_base32(lua_State *L)
 	return 1;
 }
 
-static gint
+static int
 lua_util_decode_url(lua_State *L)
 {
 	LUA_TRACE_POINT;
@@ -1324,36 +1309,36 @@ lua_util_decode_url(lua_State *L)
 }
 
 
-static gint
+static int
 lua_util_tokenize_text(lua_State *L)
 {
 	return lua_parsers_tokenize_text(L);
 }
 
-static gint
+static int
 lua_util_tanh(lua_State *L)
 {
 	LUA_TRACE_POINT;
-	gdouble in = luaL_checknumber(L, 1);
+	double in = luaL_checknumber(L, 1);
 
 	lua_pushnumber(L, tanh(in));
 
 	return 1;
 }
 
-static gint
+static int
 lua_util_parse_html(lua_State *L)
 {
 	return lua_parsers_parse_html(L);
 }
 
-static gint
+static int
 lua_util_levenshtein_distance(lua_State *L)
 {
 	LUA_TRACE_POINT;
 	struct rspamd_lua_text *t1, *t2;
-	gint dist = 0;
-	guint replace_cost = 1;
+	int dist = 0;
+	unsigned int replace_cost = 1;
 
 	t1 = lua_check_text_or_string(L, 1);
 	t2 = lua_check_text_or_string(L, 2);
@@ -1374,11 +1359,11 @@ lua_util_levenshtein_distance(lua_State *L)
 	return 1;
 }
 
-static gint
+static int
 lua_util_fold_header(lua_State *L)
 {
 	LUA_TRACE_POINT;
-	const gchar *how, *stop_chars = NULL;
+	const char *how, *stop_chars = NULL;
 	struct rspamd_lua_text *name, *value;
 	GString *folded;
 
@@ -1430,13 +1415,13 @@ lua_util_fold_header(lua_State *L)
 	return 1;
 }
 
-static gint
+static int
 lua_util_is_uppercase(lua_State *L)
 {
 	LUA_TRACE_POINT;
-	gint32 i = 0;
+	int32_t i = 0;
 	UChar32 uc;
-	guint nlc = 0, nuc = 0;
+	bool is_upper = false, is_lower = false, is_other = false;
 
 	struct rspamd_lua_text *t = lua_check_text_or_string(L, 1);
 	if (t) {
@@ -1448,15 +1433,20 @@ lua_util_is_uppercase(lua_State *L)
 			}
 
 			if (u_isupper(uc)) {
-				nuc++;
+				is_upper = true;
 			}
 			else if (u_islower(uc)) {
-				nlc++;
+				is_lower = true;
+				break;
+			}
+			else if (u_charType(uc) == U_OTHER_LETTER) {
+				is_other = true;
+				break;
 			}
 		}
 	}
 
-	if (nuc > 0 && nlc == 0) {
+	if (is_upper && !is_lower && !is_other) {
 		lua_pushboolean(L, TRUE);
 	}
 	else {
@@ -1466,12 +1456,12 @@ lua_util_is_uppercase(lua_State *L)
 	return 1;
 }
 
-static gint
+static int
 lua_util_humanize_number(lua_State *L)
 {
 	LUA_TRACE_POINT;
-	gint64 number = luaL_checkinteger(L, 1);
-	gchar numbuf[32];
+	int64_t number = luaL_checkinteger(L, 1);
+	char numbuf[32];
 
 
 	rspamd_snprintf(numbuf, sizeof(numbuf), "%hL", number);
@@ -1480,11 +1470,11 @@ lua_util_humanize_number(lua_State *L)
 	return 1;
 }
 
-static gint
+static int
 lua_util_get_tld(lua_State *L)
 {
 	LUA_TRACE_POINT;
-	const gchar *host;
+	const char *host;
 	gsize hostlen;
 	rspamd_ftok_t tld;
 
@@ -1506,13 +1496,13 @@ lua_util_get_tld(lua_State *L)
 }
 
 
-static gint
+static int
 lua_util_glob(lua_State *L)
 {
 	LUA_TRACE_POINT;
-	const gchar *pattern;
+	const char *pattern;
 	glob_t gl;
-	gint top, i, flags = 0;
+	int top, i, flags = 0;
 
 	top = lua_gettop(L);
 	memset(&gl, 0, sizeof(gl));
@@ -1533,7 +1523,7 @@ lua_util_glob(lua_State *L)
 
 	lua_createtable(L, gl.gl_pathc, 0);
 	/* Push results */
-	for (i = 0; i < (gint) gl.gl_pathc; i++) {
+	for (i = 0; i < (int) gl.gl_pathc; i++) {
 		lua_pushstring(L, gl.gl_pathv[i]);
 		lua_rawseti(L, -2, i + 1);
 	}
@@ -1543,13 +1533,13 @@ lua_util_glob(lua_State *L)
 	return 1;
 }
 
-static gint
+static int
 lua_util_parse_mail_address(lua_State *L)
 {
 	return lua_parsers_parse_mail_address(L);
 }
 
-static gint
+static int
 lua_util_strlen_utf8(lua_State *L)
 {
 	LUA_TRACE_POINT;
@@ -1558,11 +1548,11 @@ lua_util_strlen_utf8(lua_State *L)
 	t = lua_check_text_or_string(L, 1);
 
 	if (t) {
-		gint32 i = 0, nchars = 0;
+		int32_t i = 0, nchars = 0;
 		UChar32 uc;
 
 		while (i < t->len) {
-			U8_NEXT((guint8 *) t->start, i, t->len, uc);
+			U8_NEXT((uint8_t *) t->start, i, t->len, uc);
 			nchars++;
 		}
 
@@ -1575,16 +1565,16 @@ lua_util_strlen_utf8(lua_State *L)
 	return 1;
 }
 
-static gint
+static int
 lua_util_lower_utf8(lua_State *L)
 {
 	LUA_TRACE_POINT;
 	struct rspamd_lua_text *t;
 
-	gchar *dst;
+	char *dst;
 	UChar32 uc;
 	UBool err = 0;
-	gint32 i = 0, j = 0;
+	int32_t i = 0, j = 0;
 
 	t = lua_check_text_or_string(L, 1);
 
@@ -1592,7 +1582,7 @@ lua_util_lower_utf8(lua_State *L)
 		dst = g_malloc(t->len);
 
 		while (i < t->len && err == 0) {
-			U8_NEXT((guint8 *) t->start, i, t->len, uc);
+			U8_NEXT((uint8_t *) t->start, i, t->len, uc);
 			uc = u_tolower(uc);
 			U8_APPEND(dst, j, t->len, uc, err);
 		}
@@ -1614,7 +1604,7 @@ lua_util_lower_utf8(lua_State *L)
 	return 1;
 }
 
-static gint
+static int
 lua_util_normalize_utf8(lua_State *L)
 {
 	LUA_TRACE_POINT;
@@ -1647,7 +1637,7 @@ lua_util_normalize_utf8(lua_State *L)
 	return 2;
 }
 
-static gint
+static int
 lua_util_transliterate(lua_State *L)
 {
 	LUA_TRACE_POINT;
@@ -1665,12 +1655,12 @@ lua_util_transliterate(lua_State *L)
 	return 1;
 }
 
-static gint
+static int
 lua_util_strequal_caseless(lua_State *L)
 {
 	LUA_TRACE_POINT;
 	struct rspamd_lua_text *t1, *t2;
-	gint ret = -1;
+	int ret = -1;
 
 	t1 = lua_check_text_or_string(L, 1);
 	t2 = lua_check_text_or_string(L, 2);
@@ -1692,12 +1682,12 @@ lua_util_strequal_caseless(lua_State *L)
 	return 1;
 }
 
-static gint
+static int
 lua_util_strequal_caseless_utf8(lua_State *L)
 {
 	LUA_TRACE_POINT;
 	struct rspamd_lua_text *t1, *t2;
-	gint ret = -1;
+	int ret = -1;
 
 	t1 = lua_check_text_or_string(L, 1);
 	t2 = lua_check_text_or_string(L, 2);
@@ -1714,11 +1704,11 @@ lua_util_strequal_caseless_utf8(lua_State *L)
 	return 1;
 }
 
-static gint
+static int
 lua_util_get_ticks(lua_State *L)
 {
 	LUA_TRACE_POINT;
-	gdouble ticks;
+	double ticks;
 	gboolean rdtsc = FALSE;
 
 	if (lua_isboolean(L, 1)) {
@@ -1731,7 +1721,7 @@ lua_util_get_ticks(lua_State *L)
 	return 1;
 }
 
-static gint
+static int
 lua_util_get_time(lua_State *L)
 {
 	LUA_TRACE_POINT;
@@ -1741,11 +1731,11 @@ lua_util_get_time(lua_State *L)
 	return 1;
 }
 
-static gint
+static int
 lua_util_time_to_string(lua_State *L)
 {
 	LUA_TRACE_POINT;
-	gdouble seconds;
+	double seconds;
 	char timebuf[128];
 
 	if (lua_isnumber(L, 1)) {
@@ -1761,11 +1751,11 @@ lua_util_time_to_string(lua_State *L)
 	return 1;
 }
 
-static gint
+static int
 lua_util_stat(lua_State *L)
 {
 	LUA_TRACE_POINT;
-	const gchar *fpath;
+	const char *fpath;
 	struct stat st;
 
 	fpath = luaL_checkstring(L, 1);
@@ -1807,12 +1797,12 @@ lua_util_stat(lua_State *L)
 	return 2;
 }
 
-static gint
+static int
 lua_util_unlink(lua_State *L)
 {
 	LUA_TRACE_POINT;
-	const gchar *fpath;
-	gint ret;
+	const char *fpath;
+	int ret;
 
 	fpath = luaL_checkstring(L, 1);
 
@@ -1835,12 +1825,12 @@ lua_util_unlink(lua_State *L)
 	return 1;
 }
 
-static gint
+static int
 lua_util_lock_file(lua_State *L)
 {
 	LUA_TRACE_POINT;
-	const gchar *fpath;
-	gint fd = -1;
+	const char *fpath;
+	int fd = -1;
 	gboolean own = FALSE;
 
 #if !HAVE_FLOCK
@@ -1893,11 +1883,11 @@ lua_util_lock_file(lua_State *L)
 	return 1;
 }
 
-static gint
+static int
 lua_util_unlock_file(lua_State *L)
 {
 	LUA_TRACE_POINT;
-	gint fd = -1, ret, serrno;
+	int fd = -1, ret, serrno;
 	gboolean do_close = TRUE;
 
 #if !HAVE_FLOCK
@@ -1943,12 +1933,12 @@ lua_util_unlock_file(lua_State *L)
 	return 1;
 }
 
-static gint
+static int
 lua_util_create_file(lua_State *L)
 {
 	LUA_TRACE_POINT;
-	gint fd, mode = 00644;
-	const gchar *fpath;
+	int fd, mode = 00644;
+	const char *fpath;
 
 	fpath = luaL_checkstring(L, 1);
 
@@ -1975,11 +1965,11 @@ lua_util_create_file(lua_State *L)
 	return 1;
 }
 
-static gint
+static int
 lua_util_close_file(lua_State *L)
 {
 	LUA_TRACE_POINT;
-	gint fd = -1;
+	int fd = -1;
 
 	if (lua_isnumber(L, 1)) {
 		fd = lua_tointeger(L, 1);
@@ -2000,12 +1990,12 @@ lua_util_close_file(lua_State *L)
 	return 1;
 }
 
-static gint
+static int
 lua_util_random_hex(lua_State *L)
 {
 	LUA_TRACE_POINT;
-	gchar *buf;
-	gint buflen;
+	char *buf;
+	int buflen;
 
 	buflen = lua_tointeger(L, 1);
 
@@ -2021,41 +2011,41 @@ lua_util_random_hex(lua_State *L)
 	return 1;
 }
 
-static gint
+static int
 lua_util_zstd_compress(lua_State *L)
 {
 	return lua_compress_zstd_compress(L);
 }
 
-static gint
+static int
 lua_util_zstd_decompress(lua_State *L)
 {
 	return lua_compress_zstd_decompress(L);
 }
 
-static gint
+static int
 lua_util_gzip_compress(lua_State *L)
 {
 	return lua_compress_zlib_compress(L);
 }
 
-static gint
+static int
 lua_util_gzip_decompress(lua_State *L)
 {
 	return lua_compress_zlib_decompress(L, true);
 }
 
-static gint
+static int
 lua_util_inflate(lua_State *L)
 {
 	return lua_compress_zlib_decompress(L, false);
 }
 
-static gint
+static int
 lua_util_normalize_prob(lua_State *L)
 {
 	LUA_TRACE_POINT;
-	gdouble x, bias = 0.5;
+	double x, bias = 0.5;
 
 	x = lua_tonumber(L, 1);
 
@@ -2068,13 +2058,13 @@ lua_util_normalize_prob(lua_State *L)
 	return 1;
 }
 
-static gint
+static int
 lua_util_caseless_hash(lua_State *L)
 {
 	LUA_TRACE_POINT;
-	guint64 seed = 0xdeadbabe, h;
+	uint64_t seed = 0xdeadbabe, h;
 	struct rspamd_lua_text *t = NULL;
-	gint64 *r;
+	int64_t *r;
 
 	t = lua_check_text_or_string(L, 1);
 
@@ -2092,19 +2082,19 @@ lua_util_caseless_hash(lua_State *L)
 	h = rspamd_icase_hash(t->start, t->len, seed);
 	r = lua_newuserdata(L, sizeof(*r));
 	*r = h;
-	rspamd_lua_setclass(L, "rspamd{int64}", -1);
+	rspamd_lua_setclass(L, rspamd_int64_classname, -1);
 
 	return 1;
 }
 
-static gint
+static int
 lua_util_caseless_hash_fast(lua_State *L)
 {
 	LUA_TRACE_POINT;
-	guint64 seed = 0xdeadbabe, h;
+	uint64_t seed = 0xdeadbabe, h;
 	struct rspamd_lua_text *t = NULL;
 	union {
-		guint64 i;
+		uint64_t i;
 		double d;
 	} u;
 
@@ -2133,14 +2123,14 @@ lua_util_caseless_hash_fast(lua_State *L)
 	return 1;
 }
 
-static gint
+static int
 lua_util_is_utf_spoofed(lua_State *L)
 {
 	LUA_TRACE_POINT;
 	gsize l1, l2;
-	gint ret, nres = 2;
-	const gchar *s1 = lua_tolstring(L, 1, &l1),
-				*s2 = lua_tolstring(L, 2, &l2);
+	int ret, nres = 2;
+	const char *s1 = lua_tolstring(L, 1, &l1),
+			   *s2 = lua_tolstring(L, 2, &l2);
 	static USpoofChecker *spc, *spc_sgl;
 	UErrorCode uc_err = U_ZERO_ERROR;
 
@@ -2210,12 +2200,12 @@ lua_util_is_utf_spoofed(lua_State *L)
 	return nres;
 }
 
-static gint
+static int
 lua_util_is_utf_mixed_script(lua_State *L)
 {
 	LUA_TRACE_POINT;
 	gsize len_of_string;
-	const guchar *string_to_check = lua_tolstring(L, 1, &len_of_string);
+	const unsigned char *string_to_check = lua_tolstring(L, 1, &len_of_string);
 	UScriptCode last_script_code = USCRIPT_INVALID_CODE;
 	UErrorCode uc_err = U_ZERO_ERROR;
 
@@ -2265,17 +2255,17 @@ lua_util_is_utf_mixed_script(lua_State *L)
 	return 1;
 }
 
-static gint
+static int
 lua_util_get_string_stats(lua_State *L)
 {
 	LUA_TRACE_POINT;
-	gint num_of_digits = 0, num_of_letters = 0;
+	int num_of_digits = 0, num_of_letters = 0;
 	struct rspamd_lua_text *t;
 
 	t = lua_check_text_or_string(L, 1);
 
 	if (t) {
-		const gchar *p = t->start, *end = t->start + t->len;
+		const char *p = t->start, *end = t->start + t->len;
 		while (p < end) {
 			if (g_ascii_isdigit(*p)) {
 				num_of_digits++;
@@ -2302,14 +2292,14 @@ lua_util_get_string_stats(lua_State *L)
 }
 
 
-static gint
+static int
 lua_util_is_utf_outside_range(lua_State *L)
 {
 	LUA_TRACE_POINT;
-	gint ret;
+	int ret;
 	struct rspamd_lua_text *t = lua_check_text_or_string(L, 1);
-	guint32 range_start = lua_tointeger(L, 2);
-	guint32 range_end = lua_tointeger(L, 3);
+	uint32_t range_start = lua_tointeger(L, 2);
+	uint32_t range_end = lua_tointeger(L, 3);
 
 	static rspamd_lru_hash_t *validators;
 
@@ -2318,7 +2308,7 @@ lua_util_is_utf_outside_range(lua_State *L)
 	}
 
 	if (t) {
-		guint64 hash_key = (guint64) range_end << 32 || range_start;
+		uint64_t hash_key = (uint64_t) range_end << 32 || range_start;
 
 		USpoofChecker *validator = rspamd_lru_hash_lookup(validators, &hash_key, 0);
 
@@ -2326,7 +2316,7 @@ lua_util_is_utf_outside_range(lua_State *L)
 
 		if (validator == NULL) {
 			USet *allowed_chars;
-			guint64 *creation_hash_key = g_malloc(sizeof(guint64));
+			uint64_t *creation_hash_key = g_malloc(sizeof(uint64_t));
 			*creation_hash_key = hash_key;
 
 			validator = uspoof_open(&uc_err);
@@ -2359,7 +2349,7 @@ lua_util_is_utf_outside_range(lua_State *L)
 								   0, 0);
 		}
 
-		gint32 pos = 0;
+		int32_t pos = 0;
 		ret = uspoof_checkUTF8(validator, t->start, t->len, &pos,
 							   &uc_err);
 	}
@@ -2373,11 +2363,11 @@ lua_util_is_utf_outside_range(lua_State *L)
 }
 
 
-static gint
+static int
 lua_util_get_hostname(lua_State *L)
 {
 	LUA_TRACE_POINT;
-	gchar *hostbuf;
+	char *hostbuf;
 	gsize hostlen;
 
 	hostlen = sysconf(_SC_HOST_NAME_MAX);
@@ -2398,20 +2388,20 @@ lua_util_get_hostname(lua_State *L)
 	return 1;
 }
 
-static gint
+static int
 lua_util_parse_content_type(lua_State *L)
 {
 	return lua_parsers_parse_content_type(L);
 }
 
 
-static gint
+static int
 lua_util_mime_header_encode(lua_State *L)
 {
 	LUA_TRACE_POINT;
 	gsize len;
-	const gchar *hdr = luaL_checklstring(L, 1, &len);
-	gchar *encoded;
+	const char *hdr = luaL_checklstring(L, 1, &len);
+	char *encoded;
 
 	if (!hdr) {
 		return luaL_error(L, "invalid arguments");
@@ -2424,7 +2414,7 @@ lua_util_mime_header_encode(lua_State *L)
 	return 1;
 }
 
-static gint
+static int
 lua_util_is_valid_utf8(lua_State *L)
 {
 	LUA_TRACE_POINT;
@@ -2450,11 +2440,11 @@ lua_util_is_valid_utf8(lua_State *L)
 	return 1;
 }
 
-static gint
+static int
 lua_util_has_obscured_unicode(lua_State *L)
 {
 	LUA_TRACE_POINT;
-	gint32 i = 0, prev_i;
+	int32_t i = 0, prev_i;
 	UChar32 uc;
 
 	struct rspamd_lua_text *t = lua_check_text_or_string(L, 1);
@@ -2479,12 +2469,12 @@ lua_util_has_obscured_unicode(lua_State *L)
 	return 1;
 }
 
-static gint
+static int
 lua_util_readline(lua_State *L)
 {
 	LUA_TRACE_POINT;
-	const gchar *prompt = "";
-	gchar *input;
+	const char *prompt = "";
+	char *input = NULL;
 
 	if (lua_type(L, 1) == LUA_TSTRING) {
 		prompt = lua_tostring(L, 1);
@@ -2498,7 +2488,7 @@ lua_util_readline(lua_State *L)
 		replxx_history_add(rx_instance, "");
 	}
 
-	input = (gchar *) replxx_input(rx_instance, prompt);
+	input = (char *) replxx_input(rx_instance, prompt);
 
 	if (input) {
 		lua_pushstring(L, input);
@@ -2530,11 +2520,11 @@ lua_util_readline(lua_State *L)
 	return 1;
 }
 
-static gint
+static int
 lua_util_readpassphrase(lua_State *L)
 {
 	LUA_TRACE_POINT;
-	gchar test_password[8192];
+	char test_password[8192];
 	gsize r;
 
 	r = rspamd_read_passphrase(test_password, sizeof(test_password), 0, NULL);
@@ -2552,12 +2542,12 @@ lua_util_readpassphrase(lua_State *L)
 	return 1;
 }
 
-static gint
+static int
 lua_util_file_exists(lua_State *L)
 {
 	LUA_TRACE_POINT;
-	const gchar *fname = luaL_checkstring(L, 1);
-	gint serrno;
+	const char *fname = luaL_checkstring(L, 1);
+	int serrno;
 
 	if (fname) {
 		if (access(fname, R_OK) == -1) {
@@ -2577,13 +2567,13 @@ lua_util_file_exists(lua_State *L)
 	return 2;
 }
 
-static gint
+static int
 lua_util_mkdir(lua_State *L)
 {
 	LUA_TRACE_POINT;
-	const gchar *dname = luaL_checkstring(L, 1);
+	const char *dname = luaL_checkstring(L, 1);
 	gboolean recursive = FALSE;
-	gint r = -1;
+	int r = -1;
 
 	if (dname) {
 		if (lua_isboolean(L, 2)) {
@@ -2641,14 +2631,14 @@ lua_util_mkdir(lua_State *L)
 }
 
 
-static gint
+static int
 lua_util_umask(lua_State *L)
 {
 	LUA_TRACE_POINT;
 	mode_t mask = 0, old;
 
 	if (lua_type(L, 1) == LUA_TSTRING) {
-		const gchar *str = lua_tostring(L, 1);
+		const char *str = lua_tostring(L, 1);
 
 		if (str[0] == '0') {
 			/* e.g. '022' */
@@ -2673,7 +2663,7 @@ lua_util_umask(lua_State *L)
 	return 1;
 }
 
-static gint
+static int
 lua_util_isatty(lua_State *L)
 {
 	LUA_TRACE_POINT;
@@ -3426,15 +3416,15 @@ lua_util_unpack(lua_State *L)
 static int
 lua_util_btc_polymod(lua_State *L)
 {
-	guint64 c = 1;
+	uint64_t c = 1;
 
 	if (lua_type(L, 1) != LUA_TTABLE) {
 		return luaL_error(L, "invalid arguments");
 	}
 
 	for (lua_pushnil(L); lua_next(L, 1); lua_pop(L, 1)) {
-		guint8 c0 = c >> 35;
-		guint64 d = lua_tointeger(L, -1);
+		uint8_t c0 = c >> 35;
+		uint64_t d = lua_tointeger(L, -1);
 
 		c = ((c & 0x07ffffffff) << 5) ^ d;
 
@@ -3462,7 +3452,7 @@ lua_util_parse_smtp_date(lua_State *L)
 }
 
 
-static gint
+static int
 lua_load_util(lua_State *L)
 {
 	lua_newtable(L);
@@ -3471,7 +3461,7 @@ lua_load_util(lua_State *L)
 	return 1;
 }
 
-static gint
+static int
 lua_load_int64(lua_State *L)
 {
 	lua_newtable(L);
@@ -3483,9 +3473,9 @@ lua_load_int64(lua_State *L)
 
 void luaopen_util(lua_State *L)
 {
-	rspamd_lua_new_class(L, "rspamd{ev_base}", ev_baselib_m);
+	rspamd_lua_new_class(L, rspamd_ev_base_classname, ev_baselib_m);
 	lua_pop(L, 1);
-	rspamd_lua_new_class(L, "rspamd{int64}", int64lib_m);
+	rspamd_lua_new_class(L, rspamd_int64_classname, int64lib_m);
 	lua_pop(L, 1);
 	rspamd_lua_add_preload(L, "rspamd_util", lua_load_util);
 	rspamd_lua_add_preload(L, "rspamd_int64", lua_load_int64);
@@ -3494,8 +3484,8 @@ void luaopen_util(lua_State *L)
 static int
 lua_int64_tostring(lua_State *L)
 {
-	gint64 n = lua_check_int64(L, 1);
-	gchar buf[32];
+	int64_t n = lua_check_int64(L, 1);
+	char buf[32];
 	bool is_signed = false;
 
 	if (lua_isboolean(L, 2)) {
@@ -3519,7 +3509,7 @@ lua_int64_fromstring(lua_State *L)
 	struct rspamd_lua_text *t = lua_check_text_or_string(L, 1);
 
 	if (t && t->len > 0) {
-		guint64 u64;
+		uint64_t u64;
 		const char *p = t->start;
 		gsize len = t->len;
 		bool neg = false;
@@ -3544,8 +3534,8 @@ lua_int64_fromstring(lua_State *L)
 			return 2;
 		}
 
-		gint64 *i64_p = lua_newuserdata(L, sizeof(gint64));
-		rspamd_lua_setclass(L, "rspamd{int64}", -1);
+		int64_t *i64_p = lua_newuserdata(L, sizeof(int64_t));
+		rspamd_lua_setclass(L, rspamd_int64_classname, -1);
 		memcpy(i64_p, &u64, sizeof(u64));
 
 		if (neg) {
@@ -3561,8 +3551,8 @@ lua_int64_fromstring(lua_State *L)
 static int
 lua_int64_tonumber(lua_State *L)
 {
-	gint64 n = lua_check_int64(L, 1);
-	gdouble d;
+	int64_t n = lua_check_int64(L, 1);
+	double d;
 
 	d = n;
 	lua_pushinteger(L, d);
@@ -3573,8 +3563,8 @@ lua_int64_tonumber(lua_State *L)
 static int
 lua_int64_hex(lua_State *L)
 {
-	gint64 n = lua_check_int64(L, 1);
-	gchar buf[32];
+	int64_t n = lua_check_int64(L, 1);
+	char buf[32];
 
 	rspamd_snprintf(buf, sizeof(buf), "%XL", n);
 	lua_pushstring(L, buf);
