@@ -1,11 +1,11 @@
-/*-
- * Copyright 2016-2017 Vsevolod Stakhov
+/*
+ * Copyright 2024 Vsevolod Stakhov
  *
  * Licensed under the Apache License, Version 2.0 (the "License");
  * you may not use this file except in compliance with the License.
  * You may obtain a copy of the License at
  *
- *   http://www.apache.org/licenses/LICENSE-2.0
+ *    http://www.apache.org/licenses/LICENSE-2.0
  *
  * Unless required by applicable law or agreed to in writing, software
  * distributed under the License is distributed on an "AS IS" BASIS,
@@ -68,6 +68,7 @@ enum rspamd_worker_flags {
 	RSPAMD_WORKER_CONTROLLER = (1 << 6),
 	RSPAMD_WORKER_NO_TERMINATE_DELAY = (1 << 7),
 	RSPAMD_WORKER_OLD_CONFIG = (1 << 8),
+	RSPAMD_WORKER_NO_STRICT_CONFIG = (1 << 9),
 };
 
 struct rspamd_worker_accept_event {
@@ -83,7 +84,7 @@ typedef void (*rspamd_worker_term_cb)(EV_P_ ev_child *, struct rspamd_main *,
 struct rspamd_worker_heartbeat {
 	ev_timer heartbeat_ev; /**< used by main for checking heartbeats and by workers to send heartbeats */
 	ev_tstamp last_event;  /**< last heartbeat received timestamp */
-	gint64 nbeats;         /**< positive for beats received, negative for beats missed */
+	int64_t nbeats;        /**< positive for beats received, negative for beats missed */
 };
 
 enum rspamd_worker_state {
@@ -101,21 +102,21 @@ enum rspamd_worker_state {
 struct rspamd_worker {
 	pid_t pid;                                        /**< pid of worker									*/
 	pid_t ppid;                                       /**< pid of parent									*/
-	guint index;                                      /**< index number									*/
-	guint nconns;                                     /**< current connections count						*/
+	unsigned int index;                               /**< index number									*/
+	unsigned int nconns;                              /**< current connections count						*/
 	enum rspamd_worker_state state;                   /**< current worker state							*/
 	gboolean cores_throttled;                         /**< set to true if cores throttling took place		*/
-	gdouble start_time;                               /**< start time										*/
+	double start_time;                                /**< start time										*/
 	struct rspamd_main *srv;                          /**< pointer to server structure					*/
 	GQuark type;                                      /**< process type									*/
 	GHashTable *signal_events;                        /**< signal events									*/
 	struct rspamd_worker_accept_event *accept_events; /**< socket events				*/
 	struct rspamd_worker_conf *cf;                    /**< worker config data								*/
 	gpointer ctx;                                     /**< worker's specific data							*/
-	gint flags;                                       /**< worker's flags (enum rspamd_worker_flags)		*/
-	gint control_pipe[2];                             /**< control pipe. [0] is used by main process,
+	int flags;                                        /**< worker's flags (enum rspamd_worker_flags)		*/
+	int control_pipe[2];                              /**< control pipe. [0] is used by main process,
 	                                                   [1] is used by a worker			*/
-	gint srv_pipe[2];                                 /**< used by workers to request something from the
+	int srv_pipe[2];                                  /**< used by workers to request something from the
 	                                     main process. [0] - main, [1] - worker			*/
 	ev_io srv_ev;                                     /**< used by main for read workers' requests		*/
 	struct rspamd_worker_heartbeat hb;                /**< heartbeat data */
@@ -127,7 +128,7 @@ struct rspamd_worker {
 };
 
 struct rspamd_abstract_worker_ctx {
-	guint64 magic;
+	uint64_t magic;
 	/* Events base */
 	struct ev_loop *event_loop;
 	/* DNS resolver */
@@ -149,7 +150,7 @@ struct rspamd_worker_signal_handler_elt {
 };
 
 struct rspamd_worker_signal_handler {
-	gint signo;
+	int signo;
 	gboolean enabled;
 	ev_signal ev_sig;
 	struct ev_loop *event_loop;
@@ -163,9 +164,9 @@ struct rspamd_worker_signal_handler {
 struct module_s;
 
 struct module_ctx {
-	gint (*filter)(struct rspamd_task *task); /**< pointer to headers process function			*/
-	struct module_s *mod;                     /**< module pointer									*/
-	gboolean enabled;                         /**< true if module is enabled in configuration		*/
+	int (*filter)(struct rspamd_task *task); /**< pointer to headers process function			*/
+	struct module_s *mod;                    /**< module pointer									*/
+	gboolean enabled;                        /**< true if module is enabled in configuration		*/
 };
 
 #ifndef WITH_HYPERSCAN
@@ -208,7 +209,7 @@ struct module_ctx {
  * Module
  */
 typedef struct module_s {
-	const gchar *name;
+	const char *name;
 
 	int (*module_init_func)(struct rspamd_config *cfg, struct module_ctx **ctx);
 
@@ -219,10 +220,10 @@ typedef struct module_s {
 	int (*module_attach_controller_func)(struct module_ctx *ctx,
 										 GHashTable *custom_commands);
 
-	guint module_version;
-	guint64 rspamd_version;
-	const gchar *rspamd_features;
-	guint ctx_offset;
+	unsigned int module_version;
+	uint64_t rspamd_version;
+	const char *rspamd_features;
+	unsigned int ctx_offset;
 } module_t;
 
 enum rspamd_worker_socket_type {
@@ -233,13 +234,13 @@ enum rspamd_worker_socket_type {
 
 struct rspamd_worker_listen_socket {
 	const rspamd_inet_addr_t *addr;
-	gint fd;
+	int fd;
 	enum rspamd_worker_socket_type type;
 	bool is_systemd;
 };
 
 typedef struct worker_s {
-	const gchar *name;
+	const char *name;
 
 	gpointer (*worker_init_func)(struct rspamd_config *cfg);
 
@@ -247,9 +248,9 @@ typedef struct worker_s {
 
 	int flags;
 	int listen_type;
-	guint worker_version;
-	guint64 rspamd_version;
-	const gchar *rspamd_features;
+	unsigned int worker_version;
+	uint64_t rspamd_version;
+	const char *rspamd_features;
 } worker_t;
 
 /**
@@ -280,19 +281,19 @@ struct rspamd_cryptobox_library_ctx;
 
 #define MAX_AVG_TIME_SLOTS 31
 struct RSPAMD_ALIGNED(64) rspamd_avg_time {
-	guint32 cur_slot;
+	uint32_t cur_slot;
 	float avg_time[MAX_AVG_TIME_SLOTS];
 };
 /**
  * Server statistics
  */
 struct RSPAMD_ALIGNED(64) rspamd_stat {
-	guint messages_scanned;                /**< total number of messages scanned				*/
-	guint actions_stat[METRIC_ACTION_MAX]; /**< statistic for each action						*/
-	guint connections_count;               /**< total connections count						*/
-	guint control_connections_count;       /**< connections count to control interface			*/
-	guint messages_learned;                /**< messages learned								*/
-	struct rspamd_avg_time avg_time;       /**< average time stats								*/
+	unsigned int messages_scanned;                /**< total number of messages scanned				*/
+	unsigned int actions_stat[METRIC_ACTION_MAX]; /**< statistic for each action						*/
+	unsigned int connections_count;               /**< total connections count						*/
+	unsigned int control_connections_count;       /**< connections count to control interface			*/
+	unsigned int messages_learned;                /**< messages learned								*/
+	struct rspamd_avg_time avg_time;              /**< average time stats								*/
 };
 
 /**
@@ -319,6 +320,7 @@ struct rspamd_main {
 	struct roll_history *history; /**< rolling history								*/
 	struct ev_loop *event_loop;
 	ev_signal term_ev, int_ev, hup_ev, usr1_ev; /**< signals 										*/
+	ev_tstamp start_time;
 	struct rspamd_http_context *http_ctx;
 };
 
@@ -328,12 +330,12 @@ struct rspamd_main {
 struct controller_command;
 struct controller_session;
 
-typedef gboolean (*controller_func_t)(gchar **args,
+typedef gboolean (*controller_func_t)(char **args,
 									  struct controller_session *session);
 
 struct controller_session {
 	struct rspamd_worker *worker;         /**< pointer to worker structure (controller in fact) */
-	gint sock;                            /**< socket descriptor								*/
+	int sock;                             /**< socket descriptor								*/
 	struct controller_command *cmd;       /**< real command									*/
 	struct rspamd_config *cfg;            /**< pointer to config file							*/
 	GList *parts;                         /**< extracted mime parts							*/
@@ -345,15 +347,15 @@ struct controller_session {
 struct zstd_dictionary {
 	void *dict;
 	gsize size;
-	guint id;
+	unsigned int id;
 };
 
 struct rspamd_external_libs_ctx {
 	void **local_addrs;
 	struct rspamd_cryptobox_library_ctx *crypto_ctx;
 	struct ottery_config *ottery_cfg;
-	SSL_CTX *ssl_ctx;
-	SSL_CTX *ssl_ctx_noverify;
+	void *ssl_ctx;
+	void *ssl_ctx_noverify;
 	struct zstd_dictionary *in_dict;
 	struct zstd_dictionary *out_dict;
 	void *out_zstream;
@@ -365,7 +367,7 @@ struct rspamd_external_libs_ctx {
 /**
  * Register custom controller function
  */
-void register_custom_controller_command(const gchar *name,
+void register_custom_controller_command(const char *name,
 										controller_func_t handler,
 										gboolean privileged,
 										gboolean require_message);
